# -*- coding: utf-8 -*- 

require 'sidfmdlm_api.rb'

# エージェントからサーバへのデータ登録用API
class SidfmRegisterApiController < ApplicationController
  unloadable

  layout false

  # null_sessionは効かない模様。Redmine側で何かやっている？
  #protect_from_forgery :with => :null_session

  skip_before_action :check_if_login_required

  protect_from_forgery :except => [ :index, :register_host, :send_client_informations, :send_windows_client_informations ]

  before_action :authorize, :exception => [ :index, :register_host, :send_client_informations, :send_windows_client_informations ]

  # 導通確認用、認証不要
  # SIDfm Digitalのライセンス設定済か否かの情報も返す
  def index
    render :json => {
      :success => true,
      :license_enabled => Sidfm.sidfmdlm_license_apikey ? true : false }
  end

  def register_host
    ret = { :success => false }
    filtered_params = params.fetch(:params, {}).permit(:hostname, :register_apikey)
    hostname = filtered_params[:hostname]
    register_apikey = filtered_params[:register_apikey]
    ret_val= SidfmdHost.register_host_from_api(hostname, register_apikey, true)
    ret[:reason_id] = (ret_val[:reason_id] || '')
    ret[:reason] = (ret_val[:reason] || '')
    ret[:host_apikey] = ret_val[:host_apikey]
    if ret[:host_apikey] then
      ret[:hostname] = ret_val[:hostname]
      ret[:success] = true
      Rails.cache.clear
    end
    render :json => ret
  end

  ### 以下は、取得したホストAPIキーでの認証がはいるAPI

  # エージェント側のOS、パッケージ、その他アプリケーションの情報を送信
  # する。
  def send_client_informations
    ret = { :success => false }
    begin
      req_json = JSON.parse(request.body.read, :symbolize_names => true)
      if req_json.kind_of?(Hash) then
        host = SidfmdHost.auth_apikey(req_json[:hostname], req_json[:host_apikey])
        if host then
          SidfmdHost.transaction do
            host.lock!
            host.update_attribute :is_agent, true unless host.is_agent
            host.create_queue(req_json)
            ret[:success] = true
          end
          SidfmdHostView.sync_data([host.id])
        else
          ret[:message] = "Not Found Host or Key"
        end
      else
        ret[:message] = 'invalid request'
      end
    rescue Exception => e
      logger.error(e)
      ret[:message] = e.message
      ret[:backtrace] = e.backtrace.inspect
    end
    render :json => ret
  end

  # エージェント側のOS、パッケージ、その他アプリケーションの情報を送信
  # する。
  # Windows版専用。
  def send_windows_client_informations
    ret = { :success => false }
    begin
      req_json = JSON.parse(request.body.read, :symbolize_names => true)
      if req_json.kind_of?(Hash) then
        host = SidfmdHost.auth_apikey(req_json[:hostname], req_json[:host_apikey])
        if host then
          SidfmdHost.transaction do
            host.lock!
            host.update_attribute :is_agent, true unless host.is_agent
            host.create_queue_win(req_json)
            ret[:success] = true
          end
          SidfmdHostView.sync_data([host.id])
        else
          ret[:message] = "Not Found Host or Key"
        end
      else
        ret[:message] = 'invalid request'
      end
    rescue Exception => e
      logger.error(e)
      ret[:message] = e.message
      ret[:backtrace] = e.backtrace.inspect
    end
    render :json => ret
  end

  private

  def authorize
    # Redmineの認証処理を上書する。
  end

end
