#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

module SidfmUsersHelper

  # サマリー表のチェックボックス
  def create_summary_user_chkbox(user, checked)
    content = check_box_tag(
      'table[selected]',
      user.id,
      checked.include?(user.id),
      :disabled => user.admin,
      :class => user.admin ? '' : 'userSelect',
      :id => user.admin ? '' : 'table_selected' + user.id.to_s
    )
  end

  # サマリー表のログイン名
  def create_summary_user_login(login)
    content = content_tag(:span, login)
  end

  # サマリー表の名前
  def create_summary_user_name(user)
    content = content_tag(:span, "#{user.lastname} #{user.firstname}")
  end

  # サマリー表の認証方式
  def create_summary_user_authentication(authentication)
    content = content_tag(:span, authentication)
  end

  # サマリー表の権限
  def create_summary_user_authority(authority)
    content = content_tag(:span, authority)
  end

  # サマリー表のカテゴリ

  def create_summary_user_categories(user, user_sidfm_roles, user_role_id, current_user, current_user_sidfm_roles, current_user_category_ids)
    categories = user.sidfmd_categories.order('LOWER(name COLLATE "C")')
    categories = categories.where(:id => current_user_category_ids) unless current_user_sidfm_roles.include?(:admin)
    categories_size = categories.count
    content = ''
    if !user_sidfm_roles.include?(:admin) then
      if categories_size == 1 then
        cat = categories.first
        content = content_tag(
          :span,
          :class => "pl-1 pr-1 d-inline-block g-width-12em my-1 align-middle text-nowrap text-left",
          :style => "font-size: 86.67%;" #.btnに合わせる
        ) do
          concat(
            content_tag(
              :span, :class => "d-inline-flex", :style => "width:11em!important;"
            ) do
              concat(
                content_tag(
                  :span, :class => "text-truncate text-left"
                ) do
                  concat(
                    link_to(
                      cat.name,
                      "#",
                      :data => {
                        :search => "category",
                        :value => cat.id,
                        :toggle => "tooltip",
                        :title => "#{cat.desc}",
                        :placement => "right"
                      }
                    )
                  )
                end
              )
            end
          )
        end
      elsif categories_size > 1 && !user_sidfm_roles.include?(:admin) then
        content = create_summary_one_type_categories_balloon(categories)
      end
      if user_role_id != -1 && user.admin != true &&
        (current_user_sidfm_roles.include?(:admin) || (current_user_sidfm_roles.include?(:use) && user != current_user)) 
      then
        modify_button = button_tag(
          "カテゴリ変更",
          :type => "submit",
          :class => "btn btn-default btn-xs g-width-12em mb-1 align-middle text-nowrap",
          :role => "button",
          :autocomplete => "off",
          :data => {
            :toggle => "modal",
            :target => "#categoriesModal",
            :userid => "#{user.id.to_s}",
            :categoryids => "#{categories.pluck(:id).to_json}",
            :title => "カテゴリ編集 - #{user.login}"
          }
        )
        content = content ? content.concat(modify_button) : modify_button
      end
    end
    content
  end

  # サマリー表の採取ログイン日時
  def create_summary_user_last_login_on(last_login_on)
    content = content_tag(:span, last_login_on ? last_login_on.strftime('%Y/%m/%d %H:%M:%S') : '---')
  end

  # サマリー表のメール通知設定
  def create_summary_user_mail_notification(mail_notification)
    content = content_tag(
      :span,
      (mail_notification == "only_my_events" ? "有効" : "無効"),
      :class => "badge #{mail_notification == "only_my_events" ? "badge-success" : "badge-secondary"} g-font-size-0_93x font-weight-normal text-nowrap"
    )
  end

  # サマリー表のロック
  def create_summary_user_lock(user_status)
    content = user_status == User::STATUS_LOCKED ?
      content_tag(:span, "ロック中", :class => "badge badge-dark g-font-size-0_93x font-weight-normal text-nowrap") :
      "---"
  end

  # サマリー表の編集ボタン
  def create_user_edit_button(user, user_sidfm_roles, user_role_id, user_auth_id, current_user, current_user_sidfm_roles)
    content =
      if (!current_user_sidfm_roles.include?(:admin) && user.admin == true) ||
        (!current_user_sidfm_roles.include?(:use) && user != current_user) ||
        (!current_user_sidfm_roles.include?(:admin) && user_sidfm_roles.include?(:admin))
      then
        button_tag(
          '変更',
          :type => 'button',
          :class => 'btn btn-sm btn-edit mr-1 text-nowrap',
          :data => {
            :title => "ユーザ情報の編集",
            :func => 'mod'
          },
          :autocomplete => 'off',
          :disabled => 'disabled',
          :title => 'ユーザ情報の操作にはSIDfm™ VMの管理権限が必要です'
        )
      elsif current_user_sidfm_roles.include?(:use) || user == current_user then
        button_tag(
          '変更',
          :type => 'button',
          :class => 'btn btn-sm btn-edit mr-1 text-nowrap',
          :data => {
            :toggle => 'modal',
            :target => '#modal-form',
            :backdrop => 'static',
            :keyboard => 'true',
            :userid => user.id.to_s,
            :title => "ユーザ情報の編集",
            :login => user.login,
            :pw => '',
            :pwconf => '',
            :email => user.mail,
            :firstname => user.firstname,
            :lastname => user.lastname,
            :auth => user_auth_id.to_s,
            :roleid => user_role_id,
            :redmineadmin => user.admin,
            :useradmin => current_user_sidfm_roles.include?(:user_admin),
            :mailnotification => user.mail_notification == "only_my_events",
            :pwchange => user.must_change_passwd,
            :status => current_user_sidfm_roles.include?(:admin) ? user.status : '',
            :func => 'mod'
          },
          :autocomplete => 'off'
        )
      end
  end

  # サマリー表の削除ボタン
  def create_summary_user_delete_button(user, current_user, current_user_sidfm_roles)
    content =
      if current_user_sidfm_roles.include?(:user_admin) then
        form_tag(
          url_for(:controller => :sidfm_users, :action => :delete),
          :method => :post,
          :data => { :formdel => "user_#{user.id}"}
        ) do
          concat(
            hidden_field_tag('formdel[id]', user.id, :id => "formdel_id_#{user.id}")
          )
          .concat(
            if user.admin == true || user == current_user || user.login == 'sidfmadm' then
              submit_tag('削除', :class => 'btn btn-sm btn-delete text-nowrap', :disabled => 'disabled', :titie => 'このユーザは削除できません')
            elsif current_user_sidfm_roles.include?(:admin) || current_user_sidfm_roles.include?(:use) then
              button_tag(
                '削除',
                :type => 'button',
                :class => 'btn btn-sm btn-delete text-nowrap',
                :data => {
                  :conftext => "ユーザ「#{user.login}」を削除してもよろしいですか?",
                  :deluser => user.id
                }
              )
            else
              submit_tag(
                '削除',
                :class => 'btn btn-sm btn-delete text-nowrap',
                :disabled => 'disabled',
                :title => 'ユーザ情報の操作にはSIDfm™ VMの管理権限が必要です'
              )
            end
          )
        end
      else
        nil
      end
  end
 
  # サマリー表の操作
  def create_summary_user_operation(user, user_sidfm_roles, user_role_id, user_auth_id, current_user, current_user_sidfm_roles)
    content_tag(
      :div,
      :class => "d-flex justify-content-center"
    ) do
      concat( create_user_edit_button(user, user_sidfm_roles, user_role_id, user_auth_id, current_user, current_user_sidfm_roles) )
      .concat( create_summary_user_delete_button(user, current_user, current_user_sidfm_roles) )
    end
  end

end
