#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

class SidfmdCategory < ActiveRecord::Base
  belongs_to :user
  has_many :sidfmd_categories_hosts
  has_and_belongs_to_many :sidfmd_hosts
  has_and_belongs_to_many :users
  has_many :sidfmd_rules, dependent: :destroy
  has_many :sidfmd_rule_t_vulsyncs, dependent: :destroy
  has_many :sidfmd_rule_a_mail_categories, dependent: :destroy
  has_many :sidfmd_rule_cm_category_subs, dependent: :destroy
  has_many :sidfmd_regkey_cats, dependent: :destroy
  has_many :sidfmd_host_timelines, dependent: :destroy
  has_many :sidfmd_vul_timelines, dependent: :destroy
  has_many :sidfmd_ctags, dependent: :destroy

  scope :filter_user, -> { where('user_id IS NULL OR user_id = ?', User.anonymous? ? User.current : nil) }
  scope :system, -> { select(:id, :name, :desc).where(:is_system => true).order(:id) }
  scope :not_system, -> { select(:id, :name, :desc, :user_id).where(:is_system => false).order(:name, :id) }
  scope :with_hosts, -> { joins(:sidfmd_categories_hosts).distinct }
  scope :with_vuls, -> { where(:id => SidfmdCategoriesHost.enable_hosts.where(:sidfmd_host_id => SidfmdIssue.enable_hosts.select(:sidfmd_host_id).distinct).select(:sidfmd_category_id).distinct) }

  validates :hostlim, numericality: { greater_than_or_equal_to: 0, :allow_blank => true }, if: -> { respond_to?(:hostlim) }

  #カテゴリ名の最大文字数
  MAX_NAME_LENGTH = 250
  #概要の最大文字数
  MAX_DESC_LENGTH = 1000

  #hostlimのラベル
  HOSTLIM_LABEL = 'ホスト登録上限数'

  # システムカテゴリのラベル
  PublicServer = :public_server
  PrivateServer = :private_server
  SharedServer = :shared_server
  Mobile = :mobile
  Client = :client
  VirtualHostServer = :virtual_host_server
  PublicNetwork = :public_network
  PrivateNetwork = :private_network
  LABELS = [ PublicServer, PrivateServer, Mobile, Client, SharedServer, VirtualHostServer,
             PublicNetwork, PrivateNetwork ].freeze
  NAMES = [ '公開サーバ', '内部サーバ', 'モバイル', 'クライアント', '共用サーバ', '仮想化ホスト', '公開NW機器', '内部NW機器' ].map(&:freeze).freeze

  def self.label2id(label)
    idx = LABELS.index(label)
    idx ? idx + 1 : 0
  end

  def self.id2label(id)
    if id >= 1 and id <= 8 then
      LABELS[id - 1]
    else
      nil
    end
  end


  # 属するホストがあるならtrue
  def has_host?
    SidfmdCategoriesHost.where(:sidfmd_category_id => id).where(:sidfmd_host_id => SidfmdHost.enable_hosts.select(:id)).count > 0
  end

  # htmlidに関連するsidfmd_issuesを抽出し、issuesのsidfmd_host_id
  # hostidに関連するカテゴリ一覧を取得し返す。
  def self.get_categories_htmlid(htmlid)
    self.joins(:sidfmd_categories_hosts).joins(:sidfmd_categories_hosts => [ :sidfmd_host ]).joins(:sidfmd_hosts => [ :sidfmd_issues ]).where(:sidfmd_issues => { :html_id => htmlid }).order(:id)
  end

=begin
  def self.not_system(user = nil)
    scope :not_system, -> { select(:id, :name, :desc, :user_id).where(:is_system => false).order(:name, :id) }
  end
=end

end
