
class DynamicSummaryTable {
    //サマリー表のデータを取得ためのパラメータは以下のもののみ。
    //パラーメータを追加する必要がある場合はAddParamsToGetDataParamsFunctionを設定する
    //'pages[order]': $('#pages_order').val(),
    //'pages[offset]': pagenationData.offset, //$('#pages_offset')
    //'pages[per_page]': pagenationData.limit, //$('#pages_per_page')
    //'pages[filters]':  $('#pages_filters').val(),
    //'pages[checked]': $('#pages_checked').val(),
    //'pages[all_checked]': $('#pages_all_checked').val(),

    //「必須」とあるもの以外はオプション
    constructor({
        summaryTableId, //サマリー表のHTML要素のID   必須
        getDataPath = $('#get-display-summary-content-path').val(), //サマリー表のデータを取得ためのメソッドのpath  必須
        //getRowDataPath = $('#get-display-summary-rows-path').val(),
        bindSummaryTableRowEventsFunction, //サマリー表の行にある要素のイベントをbindする関数  必須
        pageName, //ページ名  必須
        allFilteredIdsId, //フィルタ後の全データのidを格納するHTML要素のID  必須
        chkboxSelector, //サマリー表の行を選択するチェックボックスのセレクタ(全選択は除く)  必須
        allChkboxSelector, //サマリー表の全データを選択をするチェックボックスのセレクタ  必須
        ChkboxValueType = 'int', //チェックボックスのvalueの型 
        bulkBtnSelector = '', //一括操作(アクションエリア)のボタンのセレクタ
        sortAreaId = 'sort-area', //サマリー表のソートをする要素の親セレクタ
        noMatchesText = '出力されるデータがありません。フィルタを設定している場合は、設定内容をご確認下さい。', //サマリー表に表示するデータがない時に表示するメッセージ
        funcBeforeInitializeTable, //テーブルの描画を1番最初に行う前に実行する関数
        AddParamsToGetDataParamsFunction, //サマリー表のデータを取得ためのパラメータにデータを追加する関数
        modifyCheckBoxRelatedContent, //チェックボックスの状態によって変更が必要な箇所を更新する関数
        modifyPageContent, //テーブルの表示データの変更に合わせてページの部分更新を行う関数
        checkUserInfoTarget = 'both' //サマリー表更新時に、ユーザの権限またはカテゴリ、もしくはその両方をチェックする
    }){
        let requiredItems = {
            summaryTableId, getDataPath, bindSummaryTableRowEventsFunction, pageName, allFilteredIdsId,
            chkboxSelector, allChkboxSelector
        }
        Object.keys(requiredItems).forEach(item => {
            if( requiredItems[item] == null ){
                throw(`${item} is not in args`);
            }
        });
        this.summaryTableId = summaryTableId;
        this.getDataPath = getDataPath;
        //this.getRowDataPath = getRowDataPath;
        this.pageName = pageName;
        this.allFilteredIdsId = allFilteredIdsId;
        this.chkboxSelector = chkboxSelector;
        this.allChkboxSelector = allChkboxSelector;
        this.ChkboxValueType = ChkboxValueType;
        this.bulkBtnSelector = bulkBtnSelector;
        this.bindSummaryTableRowEventsFunction = bindSummaryTableRowEventsFunction;
        this.sortAreaId = sortAreaId;
        this.noMatchesText = noMatchesText;
        this.funcBeforeInitializeTable = funcBeforeInitializeTable;
        this.AddParamsToGetDataParamsFunction = AddParamsToGetDataParamsFunction;
        this.modifyCheckBoxRelatedContent = modifyCheckBoxRelatedContent;
        this.modifyPageContent = modifyPageContent;
        this.checkUserInfoTarget = checkUserInfoTarget;
        this.executingProcess = null; //executeProcess関数等で、処理中のデータ(処理が複数実行されないようにするためのデータ)
        this.#init();
    };

    reloadOnFail(message){
        alert(message);
        $('#pages_all_checked').val('');
        $('#pages_checked').val('[]');
        const form = $('#pages_form');
        if( form.length > 0 ){
            form.submit();
        }else{
            location.reload(true);
        }
        return false;
    };

    //権限とカテゴリのチェック
    checkRolesAndCats(data, target = 'both'){
        if( this.SidfmRoles === undefined ) this.SidfmRoles = data.sidfm_roles;
        if( this.userCats === undefined ) this.userCats = data.user_cats;

        let isRoleDiff = false;
        //console.log(this.SidfmRoles, data.sidfm_roles)
        if( typeof this.SidfmRoles != typeof data.sidfm_roles ){
            isRoleDiff = true;
        }else{
            //権限変更の確認
            if( this.SidfmRoles && data.sidfm_roles && typeof this.SidfmRoles === 'object' ){
                if( JSON.stringify(this.SidfmRoles.slice().sort()) != JSON.stringify(data.sidfm_roles.slice().sort()) ){
                    isRoleDiff = true;
                }
            }else{
                if( String(this.SidfmRoles) != String(data.sidfm_roles) ){
                    isRoleDiff = true;
                }
            }
        }
        let isCatDiff = false;
        if( typeof this.userCats != typeof data.user_cats ){
            isCatDiff = true;
        }else{
            //カテゴリの確認
            if( this.userCats && data.user_cats && typeof this.userCats === 'object' ){
                //console.log(this.userCats.slice().sort(), data.user_cats.slice().sort());
                if( JSON.stringify(this.userCats.slice().sort()) != JSON.stringify(data.user_cats.slice().sort()) ){
                    isCatDiff = true;
                }
            }else{
                if( String(this.userCats) != String(data.user_cats) ){
                    isCatDiff = true;
                }
            }
        }
        let message = '';
        if( isRoleDiff ){ //権限からチャックすること
            if( target === 'both' || target === 'auth' ) {
                message += 'ログインユーザの権限が変更されているため、画面をリロードします。';
            }
        }else if( isCatDiff ){
            if( target === 'both' || target === 'categories' ) {
                message += 'ログインユーザのカテゴリが変更されているため、画面をリロードします。';
            }
        }
        return { message };
    }

    #init(){
        const Self = this;
        const summaryTable = $(`#${this.summaryTableId}`);
        const initPageSize = Number(get_summary_table_lines(this.pageName)) || 10;
        let initialized = false;

        const totalDataSize = { size: null };
        const changePageData = { pageNumber: null, pageSize: null };
        const getPageNumber = () => {
            //元々、ページを変更しても表示ページ番号は変更しないようになっていたため、同じようにする
            let pageNumber = changePageData.pageNumber === null ?
            Math.ceil(Number($('#pages_offset').val()) / Number($('#pages_per_page').val())) + 1 :
            changePageData.pageNumber;
            if(pageNumber < 1){
                pageNumber = 1;
            }else if(totalDataSize.size !== null){
                if( changePageData.pageSize ){
                    //ユーザがページ数/ページ番号を変えない限りここには入らない
                    if( totalDataSize.size != 0 ){
                        const max = Math.ceil(dataSize / changePageData.pageSize);
                        if( max < pageNumber ){
                            pageNumber = max;
                        }
                    }
                }
            }
            return pageNumber;
        };

        const fetchData = { data: null };

        summaryTable.bootstrapTable({
            sidePagination: 'server',
            totalField: 'total',
            dataField: 'rows',
            //locale: 'ja-JP',
            paginationPreText: '<<',
            paginationNextText: '>>',
            //paginationDetailHAlign: 'right',
            //paginationSuccessivelySize: 10,
            paginationParts: ['pageSize', 'pageList', 'pageInfoShort'],//'pageInfo',//,'pageInfoShort'],
            pagination: true,
            pageSize: initPageSize,
            paginationUseIntermediate: true,
            //cache: false,
            //totalRows: 500,
            pageNumber: getPageNumber(),
            method: 'post',
            pageList: [10, 25, 50, 100, 200],
            formatNoMatches: function (){
                return '<div class="text-center">' + Self.noMatchesText + '</div>';
            },
            formatDetailPagination: function(totalRows){
                return `合計 ${totalRows} 件`;
            },
            formatRecordsPerPage: function (pageNumber){
                return `最大 ${pageNumber} 件表示`;
            },
            ajax: function (params) {
                summaryTable.addClass('disabled-table');
                const pagenationData = JSON.parse(params.data);
                $('#pages_offset').val(pagenationData.offset);
                $('#pages_per_page').val(pagenationData.limit);
                const getDataParams = {
                    'pages[order]': $('#pages_order').val(),
                    'pages[offset]': pagenationData.offset,
                    'pages[per_page]': pagenationData.limit,
                    'pages[filters]':  $('#pages_filters').val(),
                    'pages[checked]': $('#pages_checked').val(),
                    'pages[all_checked]': $('#pages_all_checked').val(),
                }
                if( initialized ){
                    var hideWindow = summaryTable.closest('.fixed-table-body').find('.fixed-table-loading');
                    hideWindow.addClass('opacity-07');
                }else{
                    set_scroll();
                    if( Self.funcBeforeInitializeTable ) Self.funcBeforeInitializeTable(getDataParams);
                }

                if( Self.AddParamsToGetDataParamsFunction ){
                    Self.AddParamsToGetDataParamsFunction(getDataParams);
                }
                //console.log(getDataParams);
                //const startTime = performance.now()

                if( fetchData.data ) fetchData.data.abort(); //最後に投げられたリクエストのデータのみ表示する
                
                fetchData.data = $.post(Self.getDataPath, getDataParams, function(data){
                    //const endTime = performance.now();
                    //console.log(endTime - startTime);
                    //console.log(data)
                    if( data.err_message && data.err_message.length ){
                        Self.reloadOnFail(data.err_message);
                        return false;
                    }
                    const RoleOrCatsChanged = Self.checkRolesAndCats(data, Self.checkUserInfoTarget);
                    if( RoleOrCatsChanged.message ){
                        $('#pages_filters').val('{}');//検索中のデータも変更しないといけない可能性があるためフィルターもリセット
                        Self.reloadOnFail(RoleOrCatsChanged.message);
                        return false;
                    }
                    if( Self.modifyPageContent ) Self.modifyPageContent(data);
                    changePageData.pageNumber = null;
                    changePageData.pageSize = null;
                    $(`#${Self.allFilteredIdsId}`).val(JSON.stringify(data.all_ids));
                    //フィルタした結果のデータのidの数と、全データの数として表示する数が異なる場合は、api側でtotalの数とidの数を別々に設定する
                    totalDataSize.size = data.total;
                    const allSelectChkboxTh = summaryTable.find('th[data-field="chkbox"]')
                    if( data.total === 0 ){ //データがない場合はチェック不可
                        allSelectChkboxTh.hide();
                    }else{
                        allSelectChkboxTh.show();
                    }
                    delete data.all_ids;
                    params.success(data);
                    set_summary_table_lines(Self.pageName, pagenationData.limit);
                    summaryTable.removeClass('disabled-table');
                    if( initialized && hideWindow ) {
                        hideWindow.removeClass('opacity-07')
                    }
                    initialized = true;
                    $('.dropdown-menu:visible').siblings('[data-toggle="dropdown"]').click(); //clickでdropdownを閉じないと、再度開いた後に閉じなくなる
                }).fail(function (e){
                    if( e.statusText !== 'abort' ) {
                        //権限変更等で、セッションが切れている時
                        Self.redirectToLoginPageOnFail();
                        return;
                    }
                    //reloadOnFail('エラーが発生しました。画面をリロードします。');
                });
            },
            loadingTemplate: function (){
                return '<div class="p-3">' +
                    '<div class="loading-l spinner-border text-secondary" role="status">' +
                    '<span class="sr-only">Loading...</span>' +
                    '</div>' +
                    '</div>';
            },
        });

        const sortArea = $(`#${Self.sortAreaId}`);
        //tableがrenderされる直前
        summaryTable.on('pre-body.bs.table', function(){
            if( sortArea.length ){
                const sortingColumn = Self.getSummaryTableSortingItem();//$(`#${Self.sortAreaId}`).find('[data-field].sort-up, [data-field].sort-down');
                Self.setSortSummaryTableStyle(sortingColumn, Self.sortAreaId);
            }
            summaryTable.find('[data-toggle="tooltip"]').tooltip('hide');
            summaryTable.find('[data-toggle="dropdown"]').parent().dropdown('hide');
            $('.affitems-menu-show, .rule-setting-original-popover').remove();
            Self.closeVisibleModals();
        }); 

        //tableがrenderされた後
        summaryTable.on('post-body.bs.table', function(){
            fetchData.data = null;
            $('.progress-window,#progress-window').remove();
            Self.toggleAllCheckBoxAndBulkBtns();
            Self.bindSummaryTableRowEventsFunction();
            if( Self.modifyCheckBoxRelatedContent ) Self.modifyCheckBoxRelatedContent();
            if( Self.pageName !== 'products' ) Self.bindSummaryTableCheckBoxEvent();
        });

        //paginationでページが変わる時
        summaryTable.on('page-change.bs.table', function(e, num, size){
            changePageData.pageNumber = num;
            changePageData.pageSize = size;
        });

        //headerがrenderされた後
        /*summaryTable.on('post-header.bs.table', function(){
        });*/

        $(window).on('pageshow', function(){
            Self.toggleDynamicSummaryPriorityCategory();
            Self.bindSummaryTableCheckAllBoxEvent();
            if( Self.pageName !== 'products' ) Self.establishConsistencySummaryTableCheckBox(); //pages_checkedとチェックボックスの整合性をとる
        });

        // ソート
        if( sortArea.length ){
            sortArea.find('[data-field].sorting').on('click', function(){
                Self.sortSummaryTable($(this).attr('data-field'));
                if( sortArea.find('[data-sort].active').length ){
                    sortArea.find('[data-sort]').removeClass('active');
                    sortArea.find('.sort-up, .sort-down').addClass('active');
                }
            });
        }
    };

    refreshTable(){ //サマリー表の更新
        $('#pages_checked').val('[]');
        $(`#${this.summaryTableId}`).bootstrapTable('refresh');
    }

    //全選択のチェックボックスの切り替え
    toggleAllCheckBoxAndBulkBtns(){
        const allChecked = $('#pages_all_checked');
        const allSelect = $('#allSelect');
        const checkedLen = JSON.parse($('#pages_checked').val()).length;
        const AllidsLen = JSON.parse($('#' + this.allFilteredIdsId).val()).length;
        if( checkedLen && checkedLen === AllidsLen ){
            allSelect.prop('checked', true);
            allChecked.val('true');
        }else{
            allSelect.prop('checked', false);
            allChecked.val('');
        }
        const bulkBtns = (this.bulkBtnSelector ? $('#ActionArea').find(this.bulkBtnSelector) : null);
        if( bulkBtns ){
            if( !checkedLen ){
                bulkBtns.addClass('disabled').prop('disabled', true);
            }else{
                bulkBtns.removeClass('disabled').prop('disabled', false);
             }
        }
    }

    redirectToLoginPageOnFail(){
        alert(
            'エラーが発生しました。ログイン画面に移動します。' +
            '\n※ログインユーザの権限が変更されていることなどが原因で、セッションが切れている可能性があります。'
        );
        location.href = location.origin + '/' + $('#redmine_vm_login_path').val();
    }

    //権限とカテゴリを、サマリー表に関連するアクションを実行する前にチェックするメソッド
    checkRolesAndCatsBeforeAction(opts = {}){
        const Self = this;
        let target = 'both' ;
        if( opts.target ) target = opts.target;
        return new Promise((resolve, reject) => {
            $.ajax({
                type: 'GET',
                url: $('#sidfm_internal_api_get_user_roles_and_categories_path').val(),
            }).done(function(data){ //返却される値: { :sidfm_roles => @sidfm_roles, :user_cats => @user_category_ids }
                const RoleOrCatsChanged = Self.checkRolesAndCats(data, target);
                if( RoleOrCatsChanged.message ){
                    $('#pages_filters').val('{}');//検索中のデータも変更しないといけない可能性があるためフィルターもリセット
                    Self.reloadOnFail(
                        (opts.additionalAlertMessage ? opts.additionalAlertMessage + '\n' : '') +
                        RoleOrCatsChanged.message
                    );
                    reject(true);
                }else{
                    resolve(false);
                }
            }).fail(function(e){
                //console.log(e)
                if( e.statusText !== 'abort' ) {
                    Self.redirectToLoginPageOnFail();
                    reject(true);
                }/*else{
                    alert('エラーが発生したため、画面をリロードします。');
                    $('#pages_form').submit();
                }*/
                reject(true);
            });
        });
    }

    //サマリー表の行の更新
    /*
    updateSummaryTableRows(ids, isCheckNecessary = true){
        const Self = this;
        (async () => {
            const roleOrCategoryError = await Self.checkRolesAndCatsBeforeAction();
            if( roleOrCategoryError ) return;     
            const summaryTable = $(`#${Self.summaryTableId}`);
            const params = {
                ids: JSON.stringify(ids),
                category_id: $('#category_id').val(),
                idx: $('#pages_idx').val(),
                checked: ( isCheckNecessary ? $('#pages_checked').val() : '[]' ),
                filters: $('#pages_filters').val(),
            }
            $.post(Self.getRowDataPath, params, function(data){
                const updateRows = [];
                const rows = summaryTable.bootstrapTable('getData');
                if( rows[0] ){
                    if( rows[0].chkbox ){
                        for( let i = 0, len = rows.length; i < len; i++ ){
                            const chkboxElm = $(rows[i].chkbox);
                            const id = parseInt(chkboxElm.val());
                            if( data[id] ){
                                updateRows.push( { index: i, row: data[id] } );
                            }else{
                                chkboxElm.removeAttr('checked'); //removeAttrでないとchekced=trueのままになる
                                updateRows.push({
                                    index: i,
                                    row: { ...rows[i], chkbox: $('<div>').append( chkboxElm ).html() }
                                }); 
                            }
                        }
                    }
                }
                //summaryTable.bootstrapTable('updateRow', updateRows);
                Self.refreshTable();
            }).fail(function(){
                Self.reloadOnFail('エラーが発生しました。');
            });
        })();
    }*/

    /*getUpdateSummaryTableCheckedRowsFunction(){
        const updateRows = () => {
            const summaryTableChecked = $(`${this.chkboxSelector}:checked`);
            const summaryTableCheckedIds = []
            summaryTableChecked.each( (idx, elm) => {
                 summaryTableCheckedIds.push($(elm).val());
            });
            this.updateSummaryTableRows(summaryTableCheckedIds);
        }
        return  updateRows;
    }*/

    //サマリー表のデータ更新(複数データをプログレスバーで更新)
    //paramsDataのvalueは、objectならJSON.stringifyしておくこと
    executeProcessWithProgressBar({ ids, paramsData, idsParamsName, path, OneProcessSize = 500, method = 'POST' }){
        const Self = this;
        (async () => {
            const roleOrCategoryError = await Self.checkRolesAndCatsBeforeAction();
            if( roleOrCategoryError ) return;
            $('body').append(get_progress_bar());
            let percentage = 0;
            let totalSize = ids.length;
            let totalExecutedSize = 0;
            let updating = false;
            const update = () => {
                const sliceEndPoint = totalExecutedSize + OneProcessSize;
                const params = paramsData;
                params[idsParamsName] = JSON.stringify(ids.slice(totalExecutedSize, sliceEndPoint));
                if( Self.executingProcess && !updating ) return;
                Self.executingProcess = $.post(path, params, function(data) {
                    updating = true;
                    if( data.err_message ){
                        Self.reloadOnFail(data.err_message);
                        Self.executingProcess = null;
                        return false;
                    }
                    totalExecutedSize = sliceEndPoint;
                    if( totalExecutedSize >= totalSize ) totalExecutedSize = totalSize;

                    percentage = (totalExecutedSize / totalSize) * 100;
                    if( percentage >= 99 && percentage < 100 ){
                        percentage = 99;
                    }else{
                        percentage = Math.round(percentage);
                    }
                    $('#progress-bar').attr('aria-valuenow', percentage)
                                      .css('width', percentage + '%')
                                      .text(percentage + '%');
                    if( percentage === 100 ){
                        Self.refreshTable();
                        Self.executingProcess = null;
                    }else{
                        update();
                    }
                });
            };
            update();
        })();
    }

    //サマリー表のデータ更新(プレスバーなし)
    //処理するデータが少ないものに使用する
    executeProcess({ paramsData, path, method = 'POST', additionalAlertMessage = '', funcAfterUpdate }){
        const Self = this;
        (async () => {
            const roleOrCategoryError = await Self.checkRolesAndCatsBeforeAction({additionalAlertMessage});
            if( roleOrCategoryError ) return;
            if( !$('#checking-window').length ) $('body').append(create_loading_window());
            const update = () => {
                if( Self.executingProcess ) return;
                Self.executingProcess = $.post(path, paramsData, function(data) {
                    if( data.err_message ){
                        Self.reloadOnFail(data.err_message);
                        Self.executingProcess = null;
                        return false;
                    }
                    if( funcAfterUpdate ) { //funcAfterUpdateがある場合は、テーブルを更新せずに関数を実行
                        funcAfterUpdate();
                        $('#checking-window').remove();
                        Self.executingProcess = null;
                    }else{
                        Self.refreshTable();
                        Self.executingProcess = null;
                    }
                });
            };
            update();
        })();
    }

    //ソートのパラメータ取得
    getSummaryTableOrder(sortItem){
        const elm = $('#pages_order');
        const order = elm.val();
        if (order == sortItem + '_asc') {
            return (sortItem + '_desc');
        }
        return(sortItem + '_asc');
    }

    //ソートの項目名取得
    getSummaryTableSortingItem(){
        const order = $('#pages_order').val();
        return order.slice(0, order.lastIndexOf('_'));
    }

    //ソート
    setSortSummaryTableStyle(sortingField, sortAreaId){
        const order = $('#pages_order').val().split('_');
        const sortClass = (order[order.length - 1] === 'desc' ? 'sort-down' : 'sort-up');
        $(`#${sortAreaId}`).find('[data-field].sort-up, [data-field].sort-down').removeClass('sort-up sort-down');
        $(`#${sortAreaId}`).find('[data-field="' + sortingField + '"]').addClass(sortClass);
    }

    sortSummaryTable(field, sortAreaId = this.sortAreaId){
        const summaryTable = $(`#${this.summaryTableId}`);
        const order = this.getSummaryTableOrder(field);
        $('#pages_order').val(order);
        this.setSortSummaryTableStyle(field, sortAreaId);
        $(`#${this.summaryTableId}`).bootstrapTable('refresh');
    }

    //カテゴリの切り替え
    toggleDynamicSummaryPriorityCategory(){
        const Self = this;
        const toggleButton = $(`#${Self.summaryTableId} th .toggle-summary-priority-category`);
        if( !toggleButton ) return;

        const cookieName = 'sidfm_summary_prior_category';
        const system = 'system';
        const custom = 'custom';
        if( !$.cookie(cookieName) ){
            $.cookie(cookieName, system);
        }
       
        const priorCategoryParamElm = $('#summary_prior_category'); //カテゴリを非同期で取得するために使うパラメータを保存しておく要素
        priorCategoryParamElm.val($.cookie(cookieName));

        const togglePriorCategoryText = () => {
            toggleButton.text($.cookie(cookieName) === custom ? 'カスタム優先' : 'システム優先');
        };
        togglePriorCategoryText();
    
        toggleButton.on('click', function(e){
            e.stopPropagation();
            const current = $.cookie(cookieName);
            if( current === system ){
                $.cookie(cookieName, custom);
            }else{
                $.cookie(cookieName, system);
            }
            $(`#${Self.summaryTableId}`).bootstrapTable('refresh');
            togglePriorCategoryText();
            priorCategoryParamElm.val($.cookie(cookieName));
        });
    }

    //サマリー表のチェックボックスの選択
    bindSummaryTableCheckBoxEvent(){
        const Self = this;
        const allSelectChkBox = $(Self.allChkboxSelector);
        const SelectChkBox = $(Self.chkboxSelector);
        const bulkbtns = Self.bulkBtnSelector.length ? $(Self.bulkBtnSelector) : null;
        const pagesAllChecked = $('#pages_all_checked');
        const pagesChecked = $('#pages_checked');
        const valueType = Self.ChkboxValueType;

        SelectChkBox.on('change', function() {
            const checkbox = $(this);
            let id = checkbox.val();
            if( valueType === 'int' ){
                id = parseInt(id);
            }else if( valueType === 'string' ){
                id = String(id);
            }
            let checked = JSON.parse(pagesChecked.val());
            let all_ids = JSON.parse($('#' + Self.allFilteredIdsId).val());
            let all_ids_size = all_ids.length;
            if (checkbox.prop('checked')) {
                if (!checked.includes(id)) {
                    checked.push(id);
                    if (checked.length == all_ids_size) {
                        allSelectChkBox.prop('checked', true);
                        pagesAllChecked.val('true');
                    } else {
                        allSelectChkBox.prop('checked', false);
                        pagesAllChecked.val('');
                    }
                    pagesChecked.val(JSON.stringify(checked));
                }
            } else {
                const idx = checked.indexOf(id);
                if (idx == 0) {
                    checked = checked.slice(1, all_ids_size);
                } else if (idx == all_ids_size - 1) {
                    checked = checked.slice(0, all_ids_size - 1);
                } else if (idx > 0 ) {
                    var c_left = checked.slice(0, idx);
                    var c_right = checked.slice(idx + 1, all_ids_size);
                    checked = c_left.concat(c_right);
                }
                pagesChecked.val(JSON.stringify(checked));
                if (checked.length == all_ids_size) {
                    allSelectChkBox.prop('checked', true);
                    pagesAllChecked.val('true');
                } else {
                    allSelectChkBox.prop('checked', false);
                    pagesAllChecked.val('');
                }
            }
            if ( bulkbtns && bulkbtns.length ){
                if( checked.length > 0) {
                    bulkbtns.removeClass('disabled');
                    bulkbtns.prop('disabled', false);
                } else {
                    bulkbtns.addClass('disabled');
                    bulkbtns.prop('disabled', true);
                }
            }
            if( Self.modifyCheckBoxRelatedContent ) Self.modifyCheckBoxRelatedContent();
        });
    }

    //サマリー表の全チェックのチェックボックスのイベント
    bindSummaryTableCheckAllBoxEvent(){
        const Self = this;
        const allSelectChkBox = $(Self.allChkboxSelector);
        const bulkbtns = Self.bulkBtnSelector.length ? $(Self.bulkBtnSelector) : null;
        const pagesAllChecked = $('#pages_all_checked');
        const pagesChecked = $('#pages_checked');
        allSelectChkBox.on('change', function() {
            var checkbox = $(this);
            if (checkbox.prop('checked')) {
                pagesChecked.val($('#' + Self.allFilteredIdsId).val());
                pagesAllChecked.val('true');
                $(Self.chkboxSelector).prop('checked', true);
                if( JSON.parse(pagesChecked.val()).length && bulkbtns && bulkbtns.length ){
                    bulkbtns.removeClass('disabled')
                            .prop('disabled', false);
                }
            } else {
                pagesChecked.val(JSON.stringify([]));
                pagesAllChecked.val('');
                $(Self.chkboxSelector).prop('checked', false);
                if( bulkbtns && bulkbtns.length ){
                    bulkbtns.addClass('disabled')
                            .prop('disabled', true);
                }
            }
            if( Self.modifyCheckBoxRelatedContent ) Self.modifyCheckBoxRelatedContent();
        }); 
    }

    //サマリー表のチェックボックスの整合性をとる
    establishConsistencySummaryTableCheckBox(){
        const Self = this;
        const is_all_checked = $('#pages_all_checked').val();
        const allSelectChkBox = $(Self.allChkboxSelector);
        const bulkbtns = Self.bulkBtnSelector.length ? $(Self.bulkBtnSelector) : null;
        const pagesChecked = $('#pages_checked');
        if(is_all_checked === 'true'){
            var all_ids = JSON.parse($('#' + Self.allFilteredIdsId).val());
            allSelectChkBox.prop('checked', true);
            pagesChecked.val(JSON.stringify(all_ids.filter(Boolean)));
            if( JSON.parse(pagesChecked.val()).length && bulkbtns && bulkbtns.length ){
                bulkbtns.removeClass('disabled')
                        .prop('disabled', false);
            }
        }else{
            $(Self.chkboxSelector).prop('checked', false);
            allSelectChkBox.prop('checked', false);
            var checked = JSON.parse(pagesChecked.val());
            var target_elm = '';
            checked.forEach(function(id){
                target_elm += Self.chkboxSelector + '[value="' + id + '"],';
            });
            target_elm = target_elm.slice(0, -1);
            $(target_elm).prop('checked', true);
            if( bulkbtns && bulkbtns.length ){
                if( checked.length ){
                    bulkbtns.removeClass('disabled')
                            .prop('disabled', false);
                }else{
                    bulkbtns.addClass('disabled')
                            .prop('disabled', true);
                }
            }
        }
        if( Self.modifyCheckBoxRelatedContent ) Self.modifyCheckBoxRelatedContent();
    }

    //サマリー表のチェックボックスをリセット
    resetSummaryTablePagesCheckBox(){
        $('#pages_all_checked').val('');
        $('#pages_checked').val('[]');
        this.establishConsistencySummaryTableCheckBox();
    }

    //modalを閉じる
    closeVisibleModals(){
        const slideModals = $('.modal[data-slidemodal]:visible')
        if( slideModals.length ){
            slideModals.sort((a,b) => {
                if( parseInt($(a).attr('data-slidemodal')) < parseInt($(b).attr('data-slidemodal')) ){
                    return 1;
                }else{
                    return -1;
                }
            });
            slideModals.each((idx, elm) => { // slidemodalは後から表示されたものから順に閉じないと、２回目以降に開くときにmodal-contentの表示位置がおかしくなる
                elm = $(elm);
                elm.modal('hide');
             });
        }
        $('.modal:visible').modal('hide');
    }
};
