/* coding: utf-8 */
// filtter_buttons.jsを読み込む場合は、このファイルは読み込まないこと
// get_date関数を使用するため、sidfm.jsの読み込みが必要
$(function(){
    $('.filter-dropdown-menu *').on('click', function(e){
        e.stopPropagation();
    });
});

class SummaryTableFilter {

    constructor({
        dynamicSummaryTable,
        defaultFilterId = 'default-filters',
        filterButtonsAreaSelector = '#filter-buttons-area',
        unFilteredBtnClass = 'btn-default',
        filteringBtnClass = 'btn-secondary',
        selectedLinkClass = 'selected'
    }){
        if( !dynamicSummaryTable ){
            throw('dynamicSummaryTable is not in args');
        } 
        this.dynamicSummaryTable = dynamicSummaryTable; //サマリー表のobject。new DynamicSummaryTable() したものを渡す
        this.defaultFilterId = defaultFilterId; //デフォルトのフィルタがvalueに設定されているinputの要素ID。
        this.filterButtonsAreaSelector = filterButtonsAreaSelector; //フィルタボタン一覧の親要素のセレクタ
        this.unFilteredBtnClass = unFilteredBtnClass; //フィルタ中でないボタンのクラス
        this.filteringBtnClass = filteringBtnClass; //フィルタ中のボタンのクラス
        this.selectedLinkClass = selectedLinkClass; //フィルタボタンのダイアログ内の、フィルタされているリンクのクラス
        this.scrollToFilterButtonsArea = false; //フィルタ後に、フィルタボタンの一覧の場所までスクロールするか否か。フィルタ処理で指定する
        this.#init();
        //this.resetFunctions = []; //リセットするのに処理が必要なものはここに格納して、全リセットの時に実行する
    };

    #init(){
        const Self = this;
        $(`#${Self.dynamicSummaryTable.summaryTableId}`).on('post-body.bs.table', function(){
            if( Self.scrollToFilterButtonsArea ){ //フィルタ後に、フィルタボタンの一覧の場所までスクロールする フィルタボタン以外でフィルタした時にスクロールするために使用。
                $('html, body').animate(
                    { scrollTop: $(`${Self.filterButtonsAreaSelector}`).offset().top - $('#HEADnavi').outerHeight() },
                    550
                );
                Self.scrollToFilterButtonsArea = false;
            }
        });
    }

    //デフォルトのフィルタを返す
    getDefaultFilters(){
        return JSON.parse($(`#${this.defaultFilterId}`).val());
    }
    
    //検索ボタンを返す
    //filterNameを指定すると、その検索ボタンを、指定しないと全検索ボタンを返す
    getfilterButton(filterName = ''){
        const filterButton = filterName.length > 0 ?
            $(this.filterButtonsAreaSelector + ' [data-filter="' + filterName + '"]') :
            $(this.filterButtonsAreaSelector + ' [data-filter]');
        return filterButton;
    }
    
    resetFilterDropdownMenuChkbox(filterName){
        const filterBtn = this.getfilterButton(filterName);
        filterBtn.parent()
                 .find('.dropdown-menu input[type="checkbox"]')
                 .prop('checked', false);
    }
    
    resetPagesCheckedValues(){
        $('#pages_all_checked').val('');
        $('#pages_checked').val(JSON.stringify([]));
    }
    
    executeFilter(filters, filterName){
        this.dynamicSummaryTable.closeVisibleModals();
        this.resetPagesCheckedValues();
        $('#pages_filters').val(JSON.stringify(filters));
        $(`#${this.dynamicSummaryTable.summaryTableId}`).bootstrapTable('refresh', { pageNumber: 1 });
        const filterBtn = this.getfilterButton(filterName);
        const defaultFilter = JSON.parse($('#' + this.defaultFilterId).val());
        const filterItem = filters[filterName];
        const defaultFilterItem = defaultFilter[filterName];
        let isDiff = false;
        if( typeof filterItem != typeof defaultFilterItem ){
            isDiff = true;
        }else{
            if( typeof filterItem === 'object' ){
                if( JSON.stringify(filterItem) !== JSON.stringify(defaultFilterItem) ){
                    isDiff = true;
                }
            }else{
                if( String(filterItem) !== String(defaultFilterItem) ){
                    isDiff = true;
                }
            }
        }
        
        if( !isDiff ){ //リセット
            filterBtn.removeClass(this.filteringBtnClass).addClass(this.unFilteredBtnClass);
            this.resetFilterDropdownMenuChkbox(filterName)
        }else{
            filterBtn.addClass(this.filteringBtnClass).removeClass(this.unFilteredBtnClass);
        }
    }
    
    //全検索リセット
    resetAllFilters(refresh = true){
        const Self = this;
        Self.resetPagesCheckedValues();
        const defaultFilters = Self.getDefaultFilters();
        $('#pages_filters').val(JSON.stringify(defaultFilters));
        if( refresh ) $(`#${Self.dynamicSummaryTable.summaryTableId}`).bootstrapTable('refresh', { pageNumber: 1 });
        const allFilterBtns = Self.getfilterButton();
        allFilterBtns.removeClass(Self.filteringBtnClass).addClass(Self.unFilteredBtnClass);
        Object.keys(defaultFilters).forEach( filterName => {
            const filterBtn = Self.getfilterButton(filterName);
            const filterBtnParent = filterBtn.parent();
            if( filterBtnParent.find('[data-value]').length ){
                filterBtnParent.find('[data-value]').removeClass(Self.selectedLinkClass);
                filterBtnParent.find('[data-value="' + defaultFilters[filterName] + '"]').addClass(Self.selectedLinkClass);
            }
            filterBtnParent.find('input[type="checkbox"]').prop('checked', false);
            filterBtnParent.find('input[type="text"], textarea').val('');
        });
        //Self.resetFunctions.forEach( func => func() ); 
    }

    enableResetAllFilters(resetSelector){
        const Self = this;
        $(`${this.filterButtonsAreaSelector} ${resetSelector}`).on('click', function(){
            Self.resetAllFilters();
        });
    }
    
    //要対応リセット
    resetWarningFilter(filters){
        filters['warning'] = 'all';
        const warningFilterBtn = this.getfilterButton('warning');
        const warningFilterBtnParent = warningFilterBtn.parent();
        warningFilterBtnParent.find('[data-name][data-value]').removeClass(this.selectedLinkClass);
        warningFilterBtnParent.find('[data-name][data-value="all"]').addClass(this.selectedLinkClass);
        warningFilterBtn.removeClass(this.filteringBtnClass).addClass(this.unFilteredBtnClass);
    }
    
    //リンクでのフィルタ(要対応など)
    enableFilterChangeLink(linkSelector, cancelWarningItems = []){
        const Self = this;
        const defaultFilters = Self.getDefaultFilters();

        $(`${Self.filterButtonsAreaSelector} ${linkSelector}`).on('click', function(e) {
            e.preventDefault();
            const link = $(this);
            const name = link.attr('data-name');
            const value = link.attr('data-value');
            const filters = JSON.parse($('#pages_filters').val());
            filters[name] = value;
            if ( cancelWarningItems.includes(name) ) {
                Self.resetWarningFilter(filters);
            }
            const filterBtn = Self.getfilterButton(name);
            filterBtn.parent().find(`${linkSelector}`).removeClass(Self.selectedLinkClass);
            link.addClass(Self.selectedLinkClass);
            Self.executeFilter(filters, name);
            return false;
        });
    }
    
    //期間指定検索
    enableFilterDate(presetSelector, formSelector, resetSelector, itemName = 'date'){
        const Self = this;
        const defaultFilters = Self.getDefaultFilters();
 
        const beginDateField = $('#filters_search_date_begin');
        const endDateField = $('#filters_search_date_end');
        const outPeriodCheckElm = $('#filters_search_date_in_rev');
    
        $(`${Self.filterButtonsAreaSelector} ${presetSelector}`).on('click', function() {
            beginDateField.val($(this).data('begindate'));
            endDateField.val($(this).data('enddate'));
            outPeriodCheckElm.prop('checked', $(this).data('inrev') ? true : false);
            return false;
        });
     
        $(`${Self.filterButtonsAreaSelector} ${formSelector}`).on('submit', function() {
            var filters = JSON.parse($('#pages_filters').val());
            var beginDate = get_date($('#filters_search_date_begin').val());
            var endDate = get_date($('#filters_search_date_end').val());
            if( beginDate === false || endDate === false ){
                alert('日付は、西暦4桁、及び月と日のある日付を入力して下さい。\n※数字は半角で入力して下さい。');
                return false; 
            }else if( !beginDate.length && !endDate.length ){
                alert('日付を入力して下さい。');
                return false;
            }
            var inrev = $('#filters_search_date_in_rev').prop('checked');
            var v = (inrev ? 'rev' : 'in') + ':' + beginDate + ':' + endDate;
            filters[itemName] = String(v);
            Self.executeFilter(filters, itemName);
            return false;
        });
    
        $(`${Self.filterButtonsAreaSelector} ${resetSelector}`).on('click', function(){
            var filters = JSON.parse($('#pages_filters').val());
            filters[itemName] = defaultFilters[itemName];
            Self.executeFilter(filters, itemName);
            beginDateField.val('');
            endDateField.val('');
            outPeriodCheckElm.prop('checked', false);
            return false;
        });       
    }
    
    //SRI検索
    enableFilterSri(execSelector, resetSelector, cancelWarningItems = [], itemName = 'sris'){
        const Self = this;
        const defaultFilters = Self.getDefaultFilters();

        $(`${Self.filterButtonsAreaSelector} ${execSelector}`).on('click', function(){
            var filters = JSON.parse($('#pages_filters').val());
            var sris = [];
            $('#filter-statuses-menu [data-sri]:checked').each(function(){
                sris.push($(this).val());
            });
            filters[itemName] = sris;
            if ( cancelWarningItems.includes(itemName) ) {
                Self.resetWarningFilter(filters);
            }
            Self.executeFilter(filters, itemName);
            return false;
        });
    
        $(`${Self.filterButtonsAreaSelector} ${resetSelector}`).on('click', function(){
            var filters = JSON.parse($('#pages_filters').val());
            filters[itemName] = defaultFilters[itemName];
            Self.executeFilter(filters, itemName);
            return false;
        });
    }
    
    
    //CVSS検索
    enableFilterCvss(execSelector, resetSelector, cancelWarningItems = [], itemName = 'cvsses'){
        const Self = this;
        const defaultFilters = Self.getDefaultFilters();

        $(`${Self.filterButtonsAreaSelector} ${execSelector}`).on('click', function(){
            var filters = JSON.parse($('#pages_filters').val());
            var cvsses = [];
            $('#filter-cvsses-menu [data-cvss]:checked').each(function(){
                cvsses.push($(this).val());
            });
            filters[itemName] = cvsses;
            if ( cancelWarningItems.includes(itemName) ) {
                Self.resetWarningFilter(filters);
            }
            Self.executeFilter(filters, itemName);
            return false;
        });
    
        $(`${Self.filterButtonsAreaSelector} ${resetSelector}`).on('click', function(){
            var filters = JSON.parse($('#pages_filters').val());
            filters[itemName] = defaultFilters[itemName];
            Self.executeFilter(filters, itemName);
            return false;
        });
    }
    
    //CVSS指標検索
    enableFilterCvssVector(filterBtnSelector, execSelector, resetSelector, itemName = 'cvss_vector'){
        const Self = this;
        const defaultFilters = Self.getDefaultFilters();
        const FilterCvssVectorModal = $('#FilterCvssVectorModal');
 
        const toggleExecuteFilterCvssVectorButton = () => {
            var cvssV2Param = JSON.parse($('#cvss-v2-vectors-for-filter').val());
            var cvssV3Param = JSON.parse($('#cvss-v3-vectors-for-filter').val());
            var isUnregistCheckboxChecked = $('#filter-unregistered-cvss-vector-checkbox').prop('checked');
            var isSelectedValue = false;
            if( isUnregistCheckboxChecked ){
                isSelectedValue = true;
            }
            if( !isSelectedValue ){
                $.each(cvssV2Param, function(vector, value){
                    if( isSelectedValue ){
                        return false;
                    }
                    if( value ){
                        isSelectedValue = true;
                    }
                });
                $.each(cvssV3Param, function(vector, value){
                    if( isSelectedValue ){
                        return false;
                    }
                    if( value ){
                        isSelectedValue = true;
                    }
                });
            }
            if( isSelectedValue ){
                $(execSelector).prop('disabled', false);
            }else{
                $(execSelector).prop('disabled', true);
            }
        };
       
        const toggleFilterCvssAreas = (checkbox) => {
           if( checkbox.prop('checked') ){
               $('#cvss-v2-vectors-area').append(
                   $('<div>').attr('class', 'disabled-cvss-area')
               );
               $('#cvss-v3-vectors-area').append(
                   $('<div>').attr('class', 'disabled-cvss-area')
               );
           }else{
               $('#cvss-v2-vectors-area .disabled-cvss-area').remove();
               $('#cvss-v3-vectors-area .disabled-cvss-area').remove();
           }
       };
      
       $(filterBtnSelector).on('click', function(){
           var filters = JSON.parse($('#pages_filters').val());
           if( Object.keys(filters[itemName]).length > 0 ) {
               var filteredCvssV2 = filters[itemName]['cvss_v2'];
               var filteredCvssV3 = filters[itemName]['cvss_v3'];
               $('#cvss-v2-vectors-for-filter').val(JSON.stringify(filteredCvssV2));
               $('#cvss-v3-vectors-for-filter').val(JSON.stringify(filteredCvssV3));
               var cvssV2Area = $('#cvss-v2-vectors-area');
               var cvssV3Area = $('#cvss-v3-vectors-area');
               cvssV2Area.find('[data-cvss]').removeClass('selected-cvss');
               $.each(filteredCvssV2, function(vector, value){
                   if( value ){
                       cvssV2Area.find('[data-cvss="' + vector + '"][data-value="' + value + '"]').addClass('selected-cvss');
                   }
               });
               cvssV3Area.find('[data-cvss]').removeClass('selected-cvss');
               $.each(filteredCvssV3, function(vector, value){
                   if( value ){
                       cvssV3Area.find('[data-cvss="' + vector + '"][data-value="' + value + '"]').addClass('selected-cvss');
                   }
               });
               var checkbox = $('#filter-unregistered-cvss-vector-checkbox');
               if( filters[itemName]['is_target_unregistered'] ){
                   checkbox.prop('checked', true);
               }else{
                   checkbox.prop('checked', false);
               }
               toggleFilterCvssAreas(checkbox);
           }else{
               var checkbox = $('#filter-unregistered-cvss-vector-checkbox');
               checkbox.prop('checked', false);
               toggleFilterCvssAreas(checkbox);
               var defaultCvssV2Val = {};
               var cvssV2VectorItems = JSON.parse($('#cvss-v2-vectors').val());
               $.each(cvssV2VectorItems, function(vector, items) {
                   defaultCvssV2Val[vector] = null;
               });
               $('#cvss-v2-vectors-for-filter').val(JSON.stringify(defaultCvssV2Val));
               $('#cvss-v2-vectors-area').find('[data-cvss]').removeClass('selected-cvss');
    
               var defaultCvssV3Val = {};
               var cvssV3VectorItems = JSON.parse($('#cvss-v3-vectors').val());
               $.each(cvssV3VectorItems, function(vector, items) {
                   defaultCvssV3Val[vector] = null;
               });
               $('#cvss-v3-vectors-for-filter').val(JSON.stringify(defaultCvssV3Val));
               $('#cvss-v3-vectors-area').find('[data-cvss]').removeClass('selected-cvss');
           }
           toggleExecuteFilterCvssVectorButton();
           FilterCvssVectorModal.modal('show');
       });
     
       const setFilterCvssItem = (clickedElm, cvssAreaName) => {
           if( cvssAreaName === 'cvss_v2' ){
               var targetParamElm = $('#cvss-v2-vectors-for-filter');
               var targetArea = $('#cvss-v2-vectors-area');
           }else{
               var targetParamElm = $('#cvss-v3-vectors-for-filter');
               var targetArea = $('#cvss-v3-vectors-area');
           }
           var param = JSON.parse(targetParamElm.val());
           var vectorName = clickedElm.attr('data-cvss');
           var isSelected = clickedElm.hasClass('selected-cvss');
           targetArea.find('[data-cvss="' + vectorName + '"]').removeClass('selected-cvss');
           if( isSelected ){
               param[vectorName] = null;
           }else{
               clickedElm.addClass('selected-cvss');
               param[vectorName] = clickedElm.attr('data-value');
           }
           targetParamElm.val(JSON.stringify(param));
           toggleExecuteFilterCvssVectorButton();
       };
    
       $('#cvss-v2-vectors-area [data-cvss]').on('click', function(){
           setFilterCvssItem($(this), 'cvss_v2');
       });
    
       $('#cvss-v3-vectors-area [data-cvss]').on('click', function(){
           setFilterCvssItem($(this), 'cvss_v3');
       });
    
       $('#filter-unregistered-cvss-vector-checkbox').on('change', function(){
           toggleFilterCvssAreas($(this));
           toggleExecuteFilterCvssVectorButton();
       });
    
       $(execSelector).on('click', function(){
           var filters = JSON.parse($('#pages_filters').val());
           var isTargetUnregistered = $('#filter-unregistered-cvss-vector-checkbox').prop('checked');
           filters[itemName]['is_target_unregistered'] = isTargetUnregistered;
           var selectedCvssV2 = JSON.parse($('#cvss-v2-vectors-for-filter').val());
           var selectedCvssV3 = JSON.parse($('#cvss-v3-vectors-for-filter').val());
           if( isTargetUnregistered ){
               $.each(selectedCvssV2, function(vector, value){
                   selectedCvssV2[vector] = null;
               });
               $.each(selectedCvssV3, function(vector, value){
                   selectedCvssV3[vector] = null;
               });
           }
           filters[itemName]['cvss_v2'] = selectedCvssV2;
           filters[itemName]['cvss_v3'] = selectedCvssV3;
           const selectedCvssV2Values = Object.values(selectedCvssV2);
           const isselectedCvssV2 = selectedCvssV2Values.filter(v => !!v).length > 0;
           const selectedCvssV3Values = Object.values(selectedCvssV3);
           const isselectedCvssV3 = selectedCvssV3Values.filter(v => !!v).length > 0;
           Self.executeFilter(filters, itemName);
           FilterCvssVectorModal.modal('hide');
           return false;
       });
    
       $(resetSelector).on('click', function(){
           var filters = JSON.parse($('#pages_filters').val());
           filters[itemName] = defaultFilters[itemName];
           Self.executeFilter(filters, itemName);
           FilterCvssVectorModal.modal('hide');
           return false;
       });
    }
    
    //カテゴリ検索(複数)
    enableFilterMultipleCategories(execSelector, resetSelector, itemName = 'categories'){
        const Self = this;
        const defaultFilters = Self.getDefaultFilters();

        $('#filter-categories-is-and-check-box').on('change', function(){
            if( $(this).prop('checked') ){
                $('#filter-categories-is-except-check-box').prop('checked', false);
            }
        });
    
        $('#filter-categories-is-except-check-box').on('change', function(){
            if( $(this).prop('checked') ){
                $('#filter-categories-is-and-check-box').prop('checked', false);
            }
        });
    
        if( $('#filter-categories-menu #filter-categories-is-notreg-check-box').length === 1 ){
            $('#filter-categories-is-notreg-check-box').on('change', function(){
                if( $(this).prop('checked') ){
                    $('#filter-categories-is-and-check-box, #filter-categories-is-except-check-box').prop('checked', false);
                    $('#filter-summary-category-tab-content input[type="checkbox"]').prop('disabled', true);
                }else{
                    $('#filter-summary-category-tab-content input[type="checkbox"]').prop('disabled', false);
                }
            });
    
            $('#filter-categories-is-and-check-box, #filter-categories-is-except-check-box').on('change', function(){
               if( $(this).prop('checked') ){
                    $('#filter-categories-is-notreg-check-box').prop('checked', false);
                    $('#filter-summary-category-tab-content input[type="checkbox"]').prop('disabled', false);
                }
            });
        }
    
        $(`${Self.filterButtonsAreaSelector} ${execSelector}`).on('click', function(){
            var catIds = [];
            $('#filter-categories-menu .category-select:checked').each(function(){
                catIds.push($(this).val());
            });
            if( catIds.length === 0){
                if( $('#filter-categories-menu #filter-categories-is-notreg-check-box:checked').length === 0 ){
                    alert('カテゴリを1つ以上選択して下さい');
                    return false;
                }
            }
            var filters = JSON.parse($('#pages_filters').val());
            const isNotregCheckBox = $('#filter-categories-menu #filter-categories-is-notreg-check-box');
            if( isNotregCheckBox.length === 1 ){
                filters[itemName]['is_notreg'] = isNotregCheckBox.prop('checked');
            }
            if( filters[itemName]['is_notreg'] === true ){
                filters[itemName]['ids'] = [];
                filters[itemName]['is_and'] = false;
                filters[itemName]['is_except'] = false;
            }else{
                filters[itemName]['ids'] = catIds;
                filters[itemName]['is_and'] = $('#filter-categories-is-and-check-box').prop('checked');
                filters[itemName]['is_except'] = $('#filter-categories-is-except-check-box').prop('checked');
            }
            Self.executeFilter(filters, itemName);
        });
    
        (function(){
            var targetTabContent = ($.cookie('summary_category_tag') === 'custom_category' ? '#filter-summary-category-select-custom-category-tab-content' : '#filter-summary-category-select-system-category-tab-content');
            $('#filter-categories-menu li [data-target="' + targetTabContent + '"]').addClass('active');
            $(targetTabContent).show();
    
            $('#filter-categories-menu .nav-link').on('click', function(e){
                var tabItem = $(this);
                if( !tabItem.hasClass('active') ){
                    var targetContentId = tabItem.data('target');
                    $('#filter-categories-menu .nav-link, #filter-categories-menu .nav-item').removeClass('active');
                    tabItem.addClass('active');
                    tabItem.closest('.nav-item').addClass('active');
                    $('#filter-summary-category-tab-content [data-tabcontent="true"]').hide();
                    $(targetContentId).show();
                    $.cookie('summary_category_tag', (targetContentId.indexOf('system-category') === -1 ? 'custom_category' : 'system_category'), { path:'/' });
                }
                e.stopPropagation();
            });
        })();
      
        //カテゴリ検索リセット
        $(`${Self.filterButtonsAreaSelector} ${resetSelector}`).on('click', function(){
            var filters = JSON.parse($('#pages_filters').val());
            filters[itemName] = defaultFilters[itemName];
            Self.executeFilter(filters, itemName);
        });  
    }
    
    //カテゴリ検索(サマリー表内のリンクで、ページ内のデータを検索)
    enableFilterCategoryLink(linkSelector, itemName, type = 'multiple'){
        const Self = this;

        const updatePage = () => {
             Self.dynamicSummaryTable.reloadOnFail('カテゴリが更新されているため、画面をリロードします。');
        };

        $(`#${Self.dynamicSummaryTable.summaryTableId} ${linkSelector}`).on('click', function() {
            const filterName = $(this).attr('data-name');
            const categoryId = $(this).attr('data-value');
            const filters = JSON.parse($('#pages_filters').val());
            const filterBtn = Self.getfilterButton(itemName);
    
            if( type === 'multiple' ){
                //カテゴリの複数選択が対象
                if( filters[itemName]['is_notreg'] !== undefined ) filters[itemName]['is_notreg'] = false;
                if( filters[itemName]['is_and'] !== undefined ) filters[itemName]['is_and'] = false;
                if( filters[itemName]['is_except'] !== undefined ) filters[itemName]['is_except'] = false;
                filters[itemName]['ids'] = [ categoryId ];
                const targetCat = filterBtn.parent().find('.dropdown-menu input[value="' + categoryId + '"]');
                if( targetCat.length ){
                    targetCat.prop('checked', true);
                }else{
                    //カテゴリが存在しない場合は、画面をリロード
                    updatePage();
                    return false;
                }
            }else{
                //カテゴリの単数選択が対象
                filters[itemName] = categoryId;
                filterBtn.parent().find('[data-name][data-value]').removeClass(Self.selectedLinkClass);
                const targetCat = filterBtn.parent()
                                           .find('[data-name="' + itemName + '"][data-value="' + categoryId + '"]');
                if( targetCat.length ){
                    targetCat.addClass(Self.selectedLinkClass);
                }else{
                    //カテゴリが存在しない場合は、画面をリロード
                    updatePage();
                    return false;
                }
            }
            Self.executeFilter(filters, itemName);       
        });
    }

    //有効期間/対策予定日検索
    enableFilterDate2(execSelector, resetSelector, itemName = 'due_date'){
        const Self = this;
        const defaultFilters = Self.getDefaultFilters();
 
        const beginDateField = $('#filter-date2-begin-date-field');
        const endDateField = $('#filter-date2-end-date-field');
        const notsetChkbox = $('#filter-date2-notset');
        $(`${Self.filterButtonsAreaSelector} ${execSelector}`).on('click', function(){
            let filters = JSON.parse($('#pages_filters').val());
            let beginDate = get_date(beginDateField.val());
            let endDate = get_date(endDateField.val());
            const notset = notsetChkbox.prop('checked');
            if( notset ){
                beginDate = '';
                endDate = '';
            }
            if( beginDate === false || endDate === false ){
                alert('日付は、西暦4桁、及び月と日のある日付を入力して下さい。\n※数字は半角で入力して下さい。');
                return false;
            }
            filters[itemName] = String((notset ? 'notset' : '') + ':' + beginDate + ':' + endDate);
            Self.executeFilter(filters, itemName);
            if( notset ){
                beginDateField.val('');
                endDateField.val('');    
            }
            return false;
        });
    
        $('#filter-date2-preset [data-preset="true"]').on('click', function(){
            beginDateField.val($(this).attr('data-begindate'));
            endDateField.val($(this).attr('data-enddate'));
            notsetChkbox.prop('checked', ('notset' === $(this).attr('data-notset')));
        });
    
        notsetChkbox.on('change', function(){
            if( $(this).prop('checked') ) {
                beginDateField.val('');
                endDateField.val('');
            }
        });
    
        $(`${Self.filterButtonsAreaSelector} ${resetSelector}`).on('click', function(){
            let filters = JSON.parse($('#pages_filters').val());
            filters[itemName] = defaultFilters[itemName];
            Self.executeFilter(filters, itemName);
            beginDateField.val('');
            endDateField.val('');
            return false;
        });   
    }
    
    //テキスト(文字)での検索
    enableFilterText(execSelector, resetSelector, opts, itemName = 'keyword'){
       const Self = this;
       const defaultFilters = Self.getDefaultFilters();

       const { fieldSelector, notregCheckboxElmId, isExceptCheckboxElmId, isUnlimitCheckboxElmId } = opts;
       const textField = $(fieldSelector);
    
       const execFilterText = () => {
           let filters = JSON.parse($('#pages_filters').val());
           let text = textField.val();
           if( notregCheckboxElmId ){
               filters[itemName]['is_target_notreg'] = $('#' + notregCheckboxElmId).prop('checked');
               if( filters[itemName]['is_target_notreg'] ){
                   filters[itemName]['text'] = '';
               }else{
                   filters[itemName]['text'] = text;
               }
               if( !filters[itemName]['text'].length ) textField.val('');
           }else if( isExceptCheckboxElmId ){
               if( text.length === 0 ){
                   alert('1文字以上入力して下さい。');
                   return false;
               }
               filters[itemName]['text'] = text;
               filters[itemName]['is_except'] = $('#' + isExceptCheckboxElmId).prop('checked');
           }else if( isUnlimitCheckboxElmId ){
               filters[itemName]['is_target_unlimit'] = $('#' + isUnlimitCheckboxElmId).prop('checked');
               if( filters[itemName]['is_target_unlimit'] ){
                   filters[itemName]['text'] = '';
               }else{
                   filters[itemName]['text'] = text;
               }
               if( !filters[itemName]['text'].length ) textField.val('');
           }else{
               filters[itemName] = text;
           }
           Self.executeFilter(filters, itemName);
           return false;
       };
   
       const filterButton = Self.getfilterButton(itemName);

       filterButton.parent().find('form').on('submit', function() {
           execFilterText();
           return false;
       });
    
       $(`${Self.filterButtonsAreaSelector} ${execSelector}`).on('click', function(){
           execFilterText();
           return false;
       });

       filterButton.parent().find('[data-filtertext]').on('click', function(){
           textField.val($(this).attr('data-value'));
           execFilterText();
           return false;
       });
    
       $(`${Self.filterButtonsAreaSelector} ${resetSelector}`).on('click', function(){
           let filters = JSON.parse($('#pages_filters').val());
           //if( notregCheckboxElmId ){
           //    filters[itemName]['text'] = '';
           //    filters[itemName]['is_target_notreg'] = false;
           //}else{
           //    filters[itemName] = ''
           //}
           filters[itemName] = defaultFilters[itemName];
           Self.executeFilter(filters, itemName);
           textField.val('');
           return false;
       });
    }
    
    //プロダクト検索
    enableFilterProducts(execSelector, resetSelector, pageName, itemName = 'products'){
        var jqxhr = null;
        const Self = this;
        const defaultFilters = Self.getDefaultFilters();
        const FilterProductModalSelector = '#FilterProductModal';

        const TYPE_PRODUCT = 'product';
        const TYPE_VERSION = 'version';
        const TYPE_LPRODUCT = 'lproduct';
        const TYPE_LVERSION = 'lversion';
        const prodAllVersion = '全て';
        const prodVersionAll = '全バージョン';
        const versionAllNecessary = (pageName === 'hosts' || pageName === 'vulnerability');
    
        //const getResetedFilterProductsParams = () => {
        //    const filters = JSON.parse($('#pages_filters').val());
        //    const params = {};
        //    for( key in filters[itemName]){
        //        params[key] = [];
        //    }
        //    return params;
        //};
    
        const resetModal = () => {
            const selectedData = JSON.parse($('#filter-products-selected-data').val());
            Object.keys(selectedData).forEach(key => {
                selectedData[key] = [];
            });
            $('#filter-products-selected-data').val(JSON.stringify(selectedData));
            $('#filter-products-selected-list').html('');
            $('#filter-product-list > *').remove();
            $('#filter-product-input').val('');
        };
    
        const resetFilterProducts = () => {
            var filters = JSON.parse($('#pages_filters').val());
            filters[itemName] = defaultFilters[itemName];
            //filters[itemName] = getResetedFilterProductsParams();
            Self.executeFilter(filters, itemName);
            resetModal();
        };
    
        const createSelectedProdElm = (prodName, prodId, prodType, isAllVersion = false) => {
            const item = $('<div>').attr({
                'class': 'mr-1',
                'data-prodelm': 'true'
            });
            const unselectIcon = $('<i>').attr({
                'class': 'fas fa-minus-circle ml-1 toggle-minus-circle cursor-pointer',
                'data-id': prodId,
                'data-prodtype': prodType,
                'data-allversion': isAllVersion
            });
            item.append(
                $('<div>').attr('class', 'btn-selected-product').append(
                    $('<div>').attr('class', 'd-flex').append(
                        $('<span>').attr('class', 'mr-auto').text(prodName),
                        $('<span>').attr('class', 'ml-auto').append(unselectIcon)
                    )
                )
            );
    
            //プロダクトを除外する処理
            unselectIcon.on('click', function(){
                const searchData = JSON.parse($('#filter-products-selected-data').val());
                const deleteId = $(this).data('id');
                const type = $(this).attr('data-prodtype');
                const isAllVersion = $(this).data('allversion');
                if( type === 'product' ){
                    if( isAllVersion ){
                        searchData.product_all_version_ids = searchData.product_all_version_ids.filter( pid => pid != deleteId );
                    }else{
                        searchData.product_ids = searchData.product_ids.filter( pid => pid != deleteId );
                    }
                }else if( type === 'version' ){
                    searchData.version_ids = searchData.version_ids.filter( vid => vid != deleteId );
                }else if( type === 'lproduct' ){
                    if( isAllVersion ){
                        searchData.lproduct_all_version_ids = searchData.lproduct_all_version_ids.filter( lpid => lpid != deleteId );
                    }else{
                        searchData.lproduct_ids = searchData.lproduct_ids.filter( lpid => lpid != deleteId );
                    }
                }else if( type === 'lversion' ){
                    searchData.lversion_ids = searchData.lversion_ids.filter( lvid => lvid != deleteId );
                }else{
                    return false;
                }
                $(this).closest('[data-prodelm]').remove();
                $('#filter-products-selected-data').val(JSON.stringify(searchData));
            });
            return item;
        };
    
        $('#filters-search-product-btn').on('click', function(){
            (async () => {
                const roleOrCategoryError = await Self.dynamicSummaryTable.checkRolesAndCatsBeforeAction();
                if( roleOrCategoryError ) return;
                const targetElm = $('#filter-products-selected-list');
                targetElm.html('');
                var button = $(this);
                var prod_name = button.attr('data-productname');
                $('#filter-product-input').val('');
                $('#filter-product-list > *').remove();
                const filters = JSON.parse($('#pages_filters').val());
                $('#filter-products-selected-data').val(JSON.stringify(filters[itemName]));
                const searchData = filters[itemName];
                let { product_ids, version_ids, lproduct_ids, lversion_ids } = searchData;
                const params = {
                    product_ids,
                    version_ids,
                    lproduct_ids,
                    lversion_ids,
                };
                if( searchData.product_all_version_ids ){
                    params.product_ids = params.product_ids.concat(searchData.product_all_version_ids);
                }
                if( searchData.lproduct_all_version_ids ){
                    params.lproduct_ids = params.lproduct_ids.concat(searchData.lproduct_all_version_ids);
                }

                Object.keys(params).forEach(key => {
                    params[key] = JSON.stringify(params[key]);
                });
                const appendProdElm = (prodName, prodId, prodType, isAllVersion = false) => {
                    targetElm.append( createSelectedProdElm(prodName, prodId, prodType, isAllVersion) );
                };
                if( jqxhr ){
                    jqxhr.abort();
                }
                jqxhr = $.post( $('#get-products-by-ids-path').val(), params, function(data){
                    data.products.forEach( prod => {
                        const pid = prod.pid;
                        if( searchData.product_all_version_ids && searchData.product_all_version_ids.includes(pid) ){
                            appendProdElm(prod.name + '(' + prodVersionAll + ')', pid, TYPE_PRODUCT, true);
                        }
                        if( searchData.product_ids.includes(pid) ){
                            appendProdElm(prod.name + (versionAllNecessary ? '(' + prodAllVersion + ')' : ''), pid, TYPE_PRODUCT);
                        }
                    });
                    data.versions.forEach( version => {
                        const vid = version.vid;
                        appendProdElm(version.fullname, vid, TYPE_VERSION);
                    });
                    data.lproducts.forEach( lprod => {
                        const lpid = lprod.id;
                        if( searchData.lproduct_all_version_ids && searchData.lproduct_all_version_ids.includes(lpid) ){
                            appendProdElm(lprod.name + '(' + prodVersionAll + ')', lpid, TYPE_LPRODUCT, true);
                        } 
                        if( searchData.lproduct_ids.includes(lpid) ){
                            appendProdElm(lprod.name + (versionAllNecessary ?  '(' + prodAllVersion + ')' : ''), lpid, TYPE_LPRODUCT);
                        }
                    });
                    data.lversions.forEach( lversion => {
                        const lvid = lversion.id;
                        appendProdElm(lversion.fullname, lvid, TYPE_LVERSION);
                    });
                    $(FilterProductModalSelector).modal('show');
                    if( Object.values(searchData).filter( ids => ids.length > 0).length > 0 &&
                        Object.values(data).filter( ids => ids.length > 0 ).length === 0
                    ){
                        alert('検索中のプロダクトが全て削除されています。');
                        resetFilterProducts();
                    }
                });
            })();
        });
    
        $('#filter-product-input').on('input', function(){
            $('[data-toggle="tooltip"]').tooltip('hide');
            if(jqxhr){
                jqxhr.abort();
            }
            var keyword = $(this).val();
            var path = $('#sidfm_licenses_api_substr_search_path').val();
            $('#filter-product-list > *').remove();
            jqxhr = $.get(path, { 'params[keyword]': keyword }, function(data) {
                if(keyword.length < 2){
                    if(keyword.length != 0){
                        $('#filter-product-input-tooltip').attr('data-original-title', '2文字以上入力して下さい').tooltip('show');
                    }else{
                        $('#filter-product-input-tooltip').tooltip('hide');
                    }
                }else{
                    $('#filter-product-input-tooltip').tooltip('hide');
                    var lproducts = data['lproducts'];
                    delete data['lproducts'];
                    var pids = Object.keys(data);
                    if(pids.length === 0 && lproducts.length === 0){
                        $('#filter-product-input-tooltip').attr('data-original-title', '該当するプロダクトはありません').tooltip('show');
                    }else{
                        for (var i = 0; i < pids.length; i++) {
                            var product = data[pids[i]]['product'];
                            var versions = data[pids[i]]['versions'];
                            var p_item = $('<span></span>', {
                                class: 'dropdown'
                            });
                            p_item.append($('<button></button>', {
                                class: 'btn btn-outline-dark mb-1 mr-1 dropdown-toggle',
                                type: 'button',
                                'data-toggle': 'dropdown',
                                'aria-haspopup': 'true',
                                'aria-expanded': 'false',
                                text: product['name']
                             }));
                             var p_menu = $('<div></div>', {
                                 class: 'dropdown-menu scrollable-menu-30vh s-shadow-v1',
                             });
     
                             if (versions) {
                                 if( versionAllNecessary ){
                                     p_menu.append(
                                         $('<a>').attr({
                                             class: 'dropdown-item select-search-product',
                                             href: '#',
                                             'data-pid': pids[i],
                                             'data-allversion': 'true',
                                             'data-prodname': product['name'] + '( ' + prodVersionAll + ')'
                                         }).append(
                                             $('<span>').text(prodVersionAll),
                                             $('<i>').attr({
                                                 'class': 'fas fa-question-circle c_gray_70',
                                                 'data-toggle': 'tooltip',
                                                 'data-title': '関連する全てのバージョン',
                                                'data-boundary': 'window'
                                              })
                                         )
                                     );
                                 }
    
                                 const productBtn = $('<a>').attr({
                                     class: 'dropdown-item select-search-product',
                                     href: '#',
                                     'data-pid': pids[i],
                                     'data-prodname': product['name'] + (versionAllNecessary ? '( ' + prodAllVersion + ')' : ''),
                                 });
                                 productBtn.append($('<span>').text(prodAllVersion));
    
                                 if( versionAllNecessary ){
                                     productBtn.append(
                                         $('<i>').attr({
                                             'class': 'fas fa-question-circle c_gray_70',
                                             'data-toggle': 'tooltip',
                                             'data-title': 'ホストとの関連付けで「全て」を指定したもの',
                                             'data-boundary': 'window'
                                         })
                                     );
                                 }
                                 p_menu.append(productBtn);
                             }else{
                                 p_menu.append(
                                     $('<a>').attr({
                                         class: 'dropdown-item select-search-product',
                                         href: '#',
                                         'data-pid': pids[i],
                                         'data-prodname': product['name'] + (versionAllNecessary ? '( ' + prodAllVersion + ')' : ''),
                                     }).text(prodAllVersion)
                                 );
                             }
                             if (versions) {
                                 for (var j = 0; j < versions.length; j++) {
                                     var version = versions[j];
                                     var p_ver = $('<a></a>', {
                                         class: 'dropdown-item select-search-product',
                                         href: '#',
                                         text: version['fullname'],
                                         'data-vid': version['vid'],
                                         'data-prodname': version['fullname']
                                     });
                                     p_menu.append(p_ver);
                                 }
                             }
                             p_item.append(p_menu);
                             $('#filter-product-list').append(p_item);
    
                        }
                        for (var i = 0; i < lproducts.length; i++) {
                            var lproduct_data = lproducts[i];
                            var lproduct = lproduct_data['product'];
                            var lversions = lproduct_data['versions'];
                            var p_item = $('<span></span>', {
                                class: 'dropdown lproduct-dropdown'
                            });
                            p_item.append($('<button></button>', {
                                class: 'btn btn-outline-dark mb-1 mr-1 dropdown-toggle',
                                type: 'button',
                                'data-toggle': 'dropdown',
                                'aria-haspopup': 'true',
                                'aria-expanded': 'false',
                                text: lproduct['name']
                            }));
                            var p_menu = $('<div></div>', {
                                class: 'dropdown-menu scrollable-menu-30vh s-shadow-v1',
                                       'aria-labelledby': 'm_searched_item_' + lproduct['id']
                            });
                            if(lversions.length > 0){
                                if( versionAllNecessary ){
                                    p_menu.append(
                                        $('<a>').attr({
                                            class: 'dropdown-item select-search-product',
                                            href: '#',
                                            'data-lpid': lproduct['id'],
                                            'data-allversion': 'true',
                                            'data-prodname': lproduct['name'] + '( ' + prodVersionAll + ')'
                                        }).append(
                                            $('<span>').text(prodVersionAll),
                                            $('<i>').attr({
                                                'class': 'fas fa-question-circle c_gray_70',
                                                'data-toggle': 'tooltip',
                                                'data-title': '関連する全てのバージョン',
                                                'data-boundary': 'window'
                                            })
                                        )
                                    );
                                }
    
                                const lproductBtn = $('<a>').attr({
                                    class: 'dropdown-item select-search-product',
                                    href: '#',
                                    'data-lpid': lproduct['id'],
                                    'data-prodname': lproduct['name'] + (versionAllNecessary ? '( ' + prodAllVersion + ')' : ''),
                                });
                                lproductBtn.append($('<span>').text(prodAllVersion));
    
                                if( versionAllNecessary ){
                                    lproductBtn.append(
                                        $('<i>').attr({
                                            'class': 'fas fa-question-circle c_gray_70',
                                            'data-toggle': 'tooltip',
                                            'data-title': 'ホストとの関連付けで「全て」を指定したもの',
                                            'data-boundary': 'window'
                                        })
                                    );
                                }
                                p_menu.append(lproductBtn);
                            }else{
                                p_menu.append(
                                    $('<a>').attr({
                                        class: 'dropdown-item select-search-product',
                                        href: '#',
                                        'data-lpid': lproduct['id'],
                                        'data-prodname': lproduct['name'] + (versionAllNecessary ? '( ' + prodAllVersion + ')' : ''),
                                    }).text(prodAllVersion)
                                );
                            }
                            if (lversions.length > 0) {
                                for (var j = 0; j < lversions.length; j++) {
                                    var lversion = lversions[j];
                                    var p_ver = $('<a></a>', {
                                        class: 'dropdown-item select-search-product',
                                        href: '#',
                                        text: lversion['fullname'],
                                        'data-lvid': lversion['id'],
                                        'data-prodname': lversion['fullname']
                                    });
                                    p_menu.append(p_ver);
                                }
                            }
                            p_item.append(p_menu);
                            $('#filter-product-list').append(p_item);
                        }
                        $('#filter-product-list').find('[data-toggle="tooltip"]').tooltip();
                    }
                }
            });
        });
    
        //バージョンの選択
        $(document).on('click', `${FilterProductModalSelector} .select-search-product`, function(e){
            e.stopPropagation();
            const searchData = JSON.parse($('#filter-products-selected-data').val());
            const button = $(this);
            const prodName = button.attr('data-prodname');
            const pid = button.data('pid');
            const vid = button.data('vid');
            const lpid = button.data('lpid');
            const lvid = button.data('lvid');
            const isAllVersion = button.data('allversion');
    
            let item = null;
            if( pid ){
                if( isAllVersion ){
                    if( searchData.product_all_version_ids.includes(pid) ) return;
                    searchData.product_all_version_ids.push(pid);
                }else{
                    if( searchData.product_ids.includes(pid) ) return;
                    searchData.product_ids.push(pid);
                }
                item = createSelectedProdElm(prodName, pid, TYPE_PRODUCT, !!isAllVersion);
            }else if( vid ){
                if( searchData.version_ids.includes(vid) ) return;
                searchData.version_ids.push(vid);
                item = createSelectedProdElm(prodName, vid, TYPE_VERSION);
            }else if( lpid ){
                if( isAllVersion ){
                    if( searchData.lproduct_all_version_ids.includes(lpid) ) return;
                    searchData.lproduct_all_version_ids.push(lpid);
                }else{
                    if( searchData.lproduct_ids.includes(lpid) ) return;
                    searchData.lproduct_ids.push(lpid);
                }
                item = createSelectedProdElm(prodName, lpid, TYPE_LPRODUCT, !!isAllVersion);
            }else if( lvid ){
                if( searchData.lversion_ids.includes(lvid) ) return;
                searchData.lversion_ids.push(lvid);
                item = createSelectedProdElm(prodName, lvid, TYPE_LVERSION);
            }else{//通常の操作ではここには入らない
                return false;
            }
    
            $('#filter-products-selected-list').append(item);
            $('#filter-products-selected-data').val(JSON.stringify(searchData));
        });
    
        //検索
        $(execSelector).on('click', function(){
            const searchData = JSON.parse($('#filter-products-selected-data').val());
            var filters = JSON.parse($('#pages_filters').val());
            filters[itemName] = searchData;
            Self.executeFilter(filters, itemName);
            $(FilterProductModalSelector).modal('hide');
        });
    
        //検索リセット
        $(resetSelector).on('click', function(){
            resetFilterProducts();
            $(FilterProductModalSelector).modal('hide');
        });    
        
    }
    
    //履歴検索
    enableFilterJournals(execSelector, resetSelector, itemName = 'journal'){
        const Self = this;
        $('.filter-journal-preset').on('click', function(){
            $('#filters_search_journal_begin').val($(this).attr('data-begindate'));
            $('#filters_search_journal_end').val($(this).attr('data-enddate'));
            $('#filters_search_journal_in_rev').prop('checked', $(this).data('inrev') ? true : false);
            return false;
        });
    
        const execFilterJournals = () =>{
            var begindate = $('#filters_search_journal_begin').val();
            var enddate = $('#filters_search_journal_end').val();
            if( begindate == '' && enddate == '' ){
                alert('期間を指定して下さい');
                return false;
            }
            var filters = JSON.parse($('#pages_filters').val());
            var inrev = $('#filters_search_journal_in_rev').prop('checked');
            var v = (inrev ? 'rev' : 'in') + ':' + begindate + ':' + enddate;
            filters['journal']['period'] = String(v);
            filters['journal']['filter_status_change'] = $('#filter-journal-filter-status-change').prop('checked');
            filters['journal']['filter_duedate_change'] = $('#filter-journal-filter-duedate-change').prop('checked');
            Self.executeFilter(filters, itemName);
        };
    
        $('#filter-journal-form').on('submit', function(){
            execFilterJournals();
            return false;
        });
    
        $(execSelector).on('click', function() {
            execFilterJournals();
            return false;
        });
    
        $(resetSelector).on('click', function(){
            var filters = JSON.parse($('#pages_filters').val());
            filters['journal']['period'] = '::';
            filters['journal']['filter_status_change'] = false;
            filters['journal']['filter_duedate_change'] = false;
            Self.executeFilter(filters, itemName);
            $('#filters_search_journal_begin, #filters_search_journal_end').val('');
            $('#filters_search_journal_in_rev, #filter-journal-filter-status-change, #filter-journal-filter-duedate-change').prop('checked', false);
            return false;
        });
    }
    
    //権限検索
    enableFilterRoles(execSelector, resetSelector, itemName = 'roles'){
        const Self = this;
        $(execSelector).on('click', function(){
            var filters = JSON.parse($('#pages_filters').val());
            var ids = [];
            $('#filter-roles-menu .role-select:checked').each(function(){
                ids.push(parseInt($(this).val()));
            });
            if( ids.length == 0 ){
                alert('権限を1つ以上チェックして下さい');
                return false;
            }
            filters['roles']['ids'] = ids;
            filters['roles']['is_except'] = $('#filter-roles-is-except-check-box').prop('checked');
            Self.executeFilter(filters, itemName);
            return false;
        });
    
        $(resetSelector).on('click', function(){
            var filters = JSON.parse($('#pages_filters').val());
            filters['roles']['ids'] = [];
            filters['roles']['is_except'] = false;
            Self.executeFilter(filters, itemName);
        });
    }
    
    //ユーザ・ホスト検索
    enableFilteSearchItems(getDataPathId, itemNames){
        const Self = this;
        const create_filter_searched_elm = (data, is_add, ids, target_item_name) => {
            var btn_class = '';
            var btn_text = '';
            if( is_add ){
                if( ids.includes(data.id) ){
                    btn_class = 'btn-success';
                    btn_text = '選択済';
                }else{
                    btn_class = 'select btn-primary';
                    btn_text = '選択';
                }
            }else{
                btn_class = 'delete btn-danger';
                btn_text = '削除';
            }
            var btn = $('<button>').attr({
                          'type': 'button',
                          'class': 'text-nowrap toggle-select btn btn-sm ' +  btn_class,
                          'data-info': JSON.stringify(data),
                          'data-id': data.id,
                          'data-itemname': target_item_name
                      }).text(btn_text)
            var tr = $('<tr>').attr('class', 'item-row');
            if( target_item_name === 'users' ){
                tr.append(
                    $('<td>').attr({ 'class': 'login' }).append(
                        $('<div>').text(data.login),
                    ),
                    $('<td>').attr({ 'class': 'name' }).append(
                        $('<div>').text(data.lastname + ' ' + data.firstname),
                    ),
                    $('<td>').attr({ 'class': 'email' }).append(
                        $('<div>').text(data.address),
                    ),
                    $('<td>').attr({ 'class': 'ope' }).append(
                        $('<div>').append(btn)
                    )
                );
             }else if( target_item_name === 'hosts' ){
                 tr.append(
                     $('<td>').attr({ 'class': 'hostname'}).append(
                        $('<div>').attr({
                            'data-toggle': 'tooltip',
                            'data-title': 'ホスト名: ' + data.hostname + '<br/>表示名: ' + (data.ext_name ? data.ext_name : '(未設定)'),
                            'data-html': 'true',
                            'data-placement': 'left'
                        }).text(data.ext_name ? data.ext_name : data.hostname)
                    ),
                    $('<td>').append(
                        $('<div>').attr({ 'class': 'ope' }).append(btn)
                    )
                 );
             }
             return tr;
        }
    
        const bind_filter_selected_elm = (element) => {
            element.find('.delete').on('click', function(){
                var item_name = $(this).data('itemname');
                var param = JSON.parse($('#filter-' + item_name + '-param').val());
                var id = $(this).data('id');
                param.ids = param.ids.filter(function(selected_id){
                    return id != selected_id;
                });
                $('#filter-' + item_name + '-param').val(JSON.stringify(param));
                $(this).closest('.item-row').remove();
                var target_elm = $('#filter-' + item_name + '-searched-list [data-id="' + id + '"]')
                if( target_elm.length > 0 ){
                    target_elm.addClass('select btn-primary')
                              .removeClass('btn-success').text('選択');
                    bind_filter_searched_elms(item_name, target_elm);
                }
                check_is_filter_selected_elm(item_name);
            });
        }
    
        const bind_filter_searched_elms = (item_name, element) => {
            if( element ){
                var target = element
                target.unbind();
            }else{
                var target = $('#filter-' + item_name + '-searched-list .select');
            }
            target.on('click', function(){
                var item_name = $(this).data('itemname');
                var param = JSON.parse($('#filter-' + item_name + '-param').val());
                var id = $(this).data('id');
                if( param.ids.includes(id) ){
                    return false;
                }
                param.ids.push(id);
                $('#filter-' + item_name + '-param').val(JSON.stringify(param));
                var data = JSON.parse($(this).attr('data-info'));
                append_selected_elm(data, param, item_name);
                $(this).removeClass('select btn-primary').addClass('btn-success').text('選択済');
                check_is_filter_selected_elm(item_name);
            });
        }
    
        const check_is_filter_selected_elm = (item_name) => {
            var search_btn = $('.filter-modal[data-itemname="' + item_name + '"] [data-execute="search"]');
            var param = JSON.parse($('#filter-' + item_name + '-param').val());
            if( param.ids.length === 0 ){
                if( $('#no-selected-' + item_name + '-message').length === 0 ){
                    $('#search-' + item_name + '-modal-body').append(
                        $('<tr>').attr('id', 'no-selected-' + item_name + '-message').append(
                            $('<td>').attr({
                                'colspan': 4,
                                'class': 'text-danger text-center bg-white',
                            }).text('下の欄から検索して下さい')
                        )
                    );
                }
                if( item_name === 'users' ){
                    if( $('#filter-users-is-except-anonymous-checkbox').prop('checked') ){
                        search_btn.prop('disabled', false);
                    }else{
                        search_btn.prop('disabled', true);
                    }
                }else{
                    search_btn.prop('disabled', true);
                }
            }else{
                $('#no-selected-' + item_name + '-message').remove();
                search_btn.prop('disabled', false);
            }
        }
    
        var jqxhr = null;
        const search_data_for_filter = (target_item_name) => {
            var searched_list = $('#filter-' + target_item_name + '-searched-list-area');
            var searched_list_body = $('#filter-' + target_item_name + '-searched-list-body');
            searched_list_body.html('');
            $('#filter-' + target_item_name + '-searched-list').scrollTop(0);
            $('#search-filter-' + target_item_name + '-input-area').tooltip('hide');
            var input_val = $('#search-filter-' + target_item_name + '-input').val();
            if( input_val != '' ){
                if( jqxhr ){
                    jqxhr.abort();
                }
                var path = $('#' + getDataPathId).val();
                var item = $('#filter-' + target_item_name + '-search-item-select').val();
                var param = JSON.parse($('#filter-' + target_item_name + '-param').val());
                jqxhr = $.post( path, { 'str': input_val, 'item': item, 'filter_item': target_item_name }, function(items){
                    var elms = items.map(function(data){
                        return create_filter_searched_elm(data, true, param.ids, target_item_name);
                    });
                    if( elms.length === 0 ){
                        searched_list_body.html('');
                        searched_list.hide();
                        $('#search-filter-' + target_item_name + '-input-area').tooltip('show');
                    }else{
                        searched_list_body.append(elms);
                        bind_filter_searched_elms(target_item_name);
                        searched_list.show();
                        $('#search-filter-' + target_item_name + '-input-area').tooltip('hide');
                        if( target_item_name === 'hosts' ){
                            searched_list_body.find('[data-toggle="tooltip"]').tooltip();
                        }
                    }
                    check_is_filter_selected_elm(target_item_name);
                });
            }else{
                searched_list.hide();
            }
        }
    
        const append_selected_elm = (data, parameter, item_name) => {
            var elm = create_filter_searched_elm(data, false, parameter.ids, item_name);
            $('#search-' + item_name + '-modal-body').append(elm);
            bind_filter_selected_elm(elm);
            if( item_name === 'hosts' ){
                $('#search-' + item_name + '-modal-body [data-toggle="tooltip"]').tooltip();
            }
        }
    
        const open_filter_modal = (target_item_name) => {
            (async () => {
                const roleOrCategoryError = await Self.dynamicSummaryTable.checkRolesAndCatsBeforeAction();
                if( roleOrCategoryError ) return;
                $('#filter-' + target_item_name + '-searched-list-area').hide();
                const filters = JSON.parse($('#pages_filters').val());
                $('#filter-' + target_item_name + '-param').val(JSON.stringify(filters[target_item_name]));
                //$('#filter-' + target_item_name + '-param').val($('#original-filter-' + target_item_name + '-param').val());
                var param = JSON.parse($('#filter-' + target_item_name + '-param').val());
                $('#filter-' + target_item_name + '-is-except-checkbox').prop('checked', param.is_except);
                if( target_item_name === 'users' ){
                    $('#filter-users-is-except-anonymous-checkbox').prop('checked', param.is_except_anonymous);
                }
                $('#search-' + target_item_name + '-modal-body').html('');
                var path = $('#' + getDataPathId).val();
                var item = $('#filter-' + target_item_name + '-search-item-select').val();
                $.post( path, { 'ids': JSON.stringify(param.ids), 'item': item, 'filter_item': target_item_name }, function(items){
                    //検索中のデータ数が多い場合でも表示を速くきれいにするために先に表示分をappendして残りは後から非同期でappendする
                    if( items.length > 0 ){
                        let cnt = 0;
                        while( cnt < 10 ){
                            var data = items.shift();
                            if( data ){
                                append_selected_elm(data, param, target_item_name)
                            }else{
                                break;
                            }
                        }
                    }
                    check_is_filter_selected_elm(target_item_name);
                    $('#search-filter-' + target_item_name + '-input').val('');
                    $('#search-' + target_item_name + '-modal').modal('show');
                    setTimeout(function(){
                        $.each(items, function(){
                            append_selected_elm(this, param, target_item_name)
                        });
                    });
                });
            })();
        }
    
        //スクロールした時にthの下が透けるのを調節する
        //Safariだけはcssだけでは対応できず
        if( browser_ident === 'Safari' ){
            $('.filter-selected-list, .filter-searched-list').on('scroll', function(){
                var offset = $(this).offset();
                $(this).find('.toggle-select').each(function(){
                    if( $(this).offset().top < offset.top ){
                        $(this).addClass('invisible');
                    }else{
                        $(this).removeClass('invisible');
                    }
                });
            });
        }
    
        const modalSelectors = []
    
        if( itemNames.includes('users') ){
            modalSelectors.push('#search-users-modal.filter-modal');
            //ユーザ検索
            $('#filter-users-is-except-anonymous-checkbox').on('change', function(){
                check_is_filter_selected_elm('users');
            });
       
            $('#filter-users-search-button').on('click', function(){
                $('#filter-users-search-item-select').val('login');
                open_filter_modal('users');
            });
        }
    
        if( itemNames.includes('hosts') ){
            modalSelectors.push('#search-hosts-modal.filter-modal');
            //ホスト検索
            $('#filter-hosts-search-button').on('click', function(){
                $('#filter-hosts-search-item-select').val('hostname');
                open_filter_modal('hosts');
            });
        }
    
        const modalsSelector = modalSelectors.join(',');
    
        $(modalsSelector).on('shown.bs.modal', function(){
            var item_name = $(this).data('itemname');
            $('#search-' + item_name + '-users-input').focus();
        });
    
        $(modalsSelector).on('hide.bs.modal', function(){
            var item_name = $(this).data('itemname');
            $('#filter-' + item_name + '-selected-list, #filter-' + item_name + '-searched-list').scrollTop(0);
        });
    
        $(modalsSelector).find('[data-input="filter-search"]').on('input', function(){
            var item_name = $(this).data('itemname');
            search_data_for_filter(item_name);
        });
    
        $(modalsSelector).find('.filter-search-item-select').on('change', function(){
            var item_name = $(this).data('itemname');
            search_data_for_filter(item_name);
        });
    
        //検索modalの検索実行
        $(modalsSelector).find('[data-execute="search"]').on('click', function(){
            var item_name = $(this).data('itemname');
            var filters = JSON.parse($('#pages_filters').val());
            var param = JSON.parse($('#filter-' + item_name + '-param').val());
            filters[item_name]['ids'] = param.ids;
            filters[item_name]['is_except'] = $('#filter-' + item_name + '-is-except-checkbox').prop('checked');
            if( item_name === 'users' ){
                filters[item_name]['is_except_anonymous'] = $('#filter-users-is-except-anonymous-checkbox').prop('checked');
                param.is_except_anonymous = filters[item_name]['is_except_anonymous'];
            }
            param.is_except = filters[item_name]['is_except'];
            $('#filter-' + item_name + '-param').val(JSON.stringify(param));
            Self.executeFilter(filters, item_name);
        });
     
        //検索modalの検索リセット
        $(modalsSelector).find('[data-reset="search"]').on('click', function(){
            var item_name = $(this).data('itemname');
            var filters = JSON.parse($('#pages_filters').val());
            filters[item_name]['ids'] = [];
            filters[item_name]['is_except'] = false;
            if( item_name === 'users' ){
                filters[item_name]['is_except_anonymous'] = false;
            }
            $('#filter-' + item_name + '-param').val(JSON.stringify(filters[item_name]));
            Self.executeFilter(filters, item_name);
        });
    }
    
    //操作検索
    enableFilterOperations(execSelector, resetSelector, itemName = 'opes'){
        const Self = this;
        $(execSelector).on('click', function(){
            var filters = JSON.parse($('#pages_filters').val());
            var targets = [];
            $('#filter-opes-menu .ope-select:checked').each(function(){
                targets.push($(this).val());
            });
            if( targets.length === 0 ){
               alert('検索対象を1つ以上選択して下さい。');
               return false;
            }
            filters[itemName]['targets'] = targets;
            filters[itemName]['is_except'] = $('#filter-opes-is-except-check-box').prop('checked');
            Self.executeFilter(filters, itemName);
        });
    
        $(resetSelector).on('click', function(){
            var filters = JSON.parse($('#pages_filters').val());
            filters[itemName]['targets'] = [];
            filters[itemName]['is_except'] = false;
            Self.executeFilter(filters, itemName);
        });
    }

    //カスタムタグ検索
    enableFilterCtags(execSelector, resetSelector, itemName = 'ctags'){
        const Self = this;

        const modal = $('#filterCtagsModal');
        const searchPath = $('#filter-search-ctags-path').val();
        const getCtagPath = $('#filter-api-get-ctags-path').val();
        const selectedCtagsList = $('#filter-ctags-selected-ctags-list');
        const selectedCtagsListIds = $('#filter-ctags-selected-ctags-ids');
        const searchedCtagsListArea = $('#filter-ctags-searched-list-area');
        const searchedCtagsList = $('#filter-ctags-searched-list');
        const input = $('#filter-ctags-input');
        const inputTooltip = $('#filter-ctags-input-tooltip');
        const SearchedCtagsInfo = { fetching: false };

        //選択するとき
        const bindSelectEvent = (targetElm = null) => {
            (targetElm ? targetElm : searchedCtagsList.find('[data-select="ctag"]')).on('click', function(){
                const tag = $(this);
                const id = tag.data('id');
                const selectedData = JSON.parse(selectedCtagsListIds.val());
                if( selectedData.find(cId => cId == id) ){
                    alert('選択済みのカスタムタグです。');
                    return;
                }
                selectedData.push(id);
                selectedCtagsListIds.val(JSON.stringify(selectedData));
                const cpTag = tag.clone();
                selectedCtagsList.append(cpTag);
                bindUnselectEvent(cpTag);
           });
        }

        //選択を外すとき
        const bindUnselectEvent = (targetElm = null) => {
            (targetElm ? targetElm : selectedCtagsList.find('[data-select="ctag"]')).on('click', function(){
                const tag = $(this);
                const id = tag.data('id');
                const selectedData = JSON.parse(selectedCtagsListIds.val());
                selectedCtagsListIds.val(JSON.stringify(selectedData.filter(ctagId => ctagId != id)));
                const cpTag = tag.clone();
                tag.remove();
            });
        }

        const createTagElm = (ctag) => {
            const styleProps = [];
            styleProps.push('background-color: ' + (ctag.bgcolor ? ctag.bgcolor : '#fff') + ';');
            if( ctag.color ){
                styleProps.push('color: ' + ctag.color + ';');
            }
            return(
                $('<span>').attr({
                    class: 'searched-ctag ml-2 mr-2 mb-1 mt-1',
                    'data-id': ctag.id,
                    'data-select': 'ctag',
                    'data-name': ctag.name,
                    'style': styleProps.join(' ')
                }).text(ctag.name)
            );
        };
    
        input.on('input', function(){ //タグの検索
            const keyword = $(this).val();
            if( SearchedCtagsInfo.fetching ){
                SearchedCtagsInfo.fetching.abort();
            }
            const paramsData = { keyword };
            SearchedCtagsInfo.fetching = $.post(
                searchPath,
                paramsData,
                function(ctags){
                    const ctagElms = ctags.map( ctag => createTagElm(ctag) );
                    searchedCtagsList.html('');
                    if( ctagElms.length ){
                        searchedCtagsListArea.show();
                        inputTooltip.tooltip('hide');
                        searchedCtagsList.append(ctagElms);
                        bindSelectEvent();
                    }else{
                        searchedCtagsListArea.hide();
                        inputTooltip.tooltip(keyword.length ? 'show' : 'hide');
                    }
                }
            );
        });

        $('#filters-search-ctag-btn').on('click', function(){
            input.val('');
            searchedCtagsListArea.hide();
            const filters = JSON.parse($('#pages_filters').val());
            (async () => {
                const roleOrCategoryError = await Self.dynamicSummaryTable.checkRolesAndCatsBeforeAction();
                if( roleOrCategoryError ) return; 
                const paramsData = { ids: (filters[itemName] ? JSON.stringify(filters[itemName]['ids']) : '[]') };
                $.post(getCtagPath, paramsData, function(ctags){
                    searchedCtagsList.html('');
                    selectedCtagsList.html('');
                    const ctagElms = [];
                    const ctagIds = []; 
                    ctags.forEach( ctag => {
                        ctagElms.push(createTagElm(ctag));
                        ctagIds.push(ctag.id);
                    }); 
                    if( ctagElms.length ){
                        selectedCtagsList.append(ctagElms);
                        bindUnselectEvent();
                    }
                    selectedCtagsListIds.val(JSON.stringify(ctagIds));
                });
                modal.modal('show');
            })();
        });

        $(execSelector).on('click', function(){
            const filters = JSON.parse($('#pages_filters').val());
            const selectedIds = JSON.parse(selectedCtagsListIds.val());
            /*if( selectedIds.length === 0 ){
               alert('カスタムタグを1つ以上選択して下さい。');
               return false;
            }*/
            filters[itemName]['ids'] = selectedIds;
            Self.executeFilter(filters, itemName);
        });

        $(resetSelector).on('click', function(){
            var filters = JSON.parse($('#pages_filters').val());
            filters[itemName]['ids'] = [];
            Self.executeFilter(filters, itemName);
        });
    }

    // SRI/CVSS別分布 ・　対応の状況別分布　で使うフィルタ
    enableDisributionLink(selector, ParentSelector){
        const Self = this;

        $(`${ParentSelector} ${selector}`).on('click', function(e) {
            e.preventDefault();
            const elm = $(this);
            const name = elm.attr('data-name');
            if( name == 'sris' || name == 'cvsses' || name == 'status' ){
                Self.resetAllFilters(false);
                const value = elm.data('value');
                const filterBtn = Self.getfilterButton(name);
                const filterBtnParent = filterBtn.parent();
                let filterStatus;
                const filters = JSON.parse($('#pages_filters').val());
                /*if ( cancelWarningItems.includes(name) ) {
                    Self.resetWarningFilter(filters);
                }*/
                if (name == 'sris' || name == 'cvsses') {
                    filters[name] = value;
                    filterStatus = '1:2';
                    if( name == 'sris' ){
                        value.forEach( v => filterBtnParent.find('[data-sri][value="' + v + '"]').prop('checked', true) );
                    }else{
                        value.forEach( v => filterBtnParent.find('[data-cvss][value="' + v + '"]').prop('checked', true) );
                    }
                }else{
                    filterStatus = String(value);
                }
                filters['status'] = filterStatus;
                const statusFilterBtn = Self.getfilterButton('status');
                statusFilterBtn.parent().find('[data-name="status"]').removeClass(Self.selectedLinkClass);
                statusFilterBtn.parent().find('[data-name="status"][data-value="' + filterStatus + '"]')
                                        .addClass(Self.selectedLinkClass);
                Self.resetPagesCheckedValues();
                $('#pages_filters').val(JSON.stringify(filters));
                Self.scrollToFilterButtonsArea = true;
                $(`#${Self.dynamicSummaryTable.summaryTableId}`).bootstrapTable('refresh', { pageNumber: 1 });
                [ filterBtn, statusFilterBtn ].forEach( btn => {
                    btn.addClass(Self.filteringBtnClass).removeClass(Self.unFilteredBtnClass);
                });
            }
            return false;
        });
    }

    //危険度の分布グラフ内クリックでのフィルタ
    enableGraphRiskFilter(selector, targetIdx){
        const Self = this;
        $(`${selector}`).each((idx, element) => {
            const elm = $(element);
            const href = elm.attr('xlink:href');
            if ( href && href.startsWith('#') ) {
                elm.on('click', function(e) {
                    e.preventDefault();
                    Self.resetAllFilters(false);
                    const filterBtn = Self.getfilterButton(targetIdx == 'sri' ? 'sris' : 'cvsses');
                    const filterBtnParent = filterBtn.parent();
                    const index = $(this).attr('xlink:href').slice(1);
                    const filters = JSON.parse($('#pages_filters').val());
                    let filterIndex = false;
                    if( targetIdx == 'sri' ){
                        if (index >= 0 && index <= 4) {
                            filters['status'] = '1:2';
                            filters['sris'] = [ index ];
                            filterBtnParent.find('[data-sri][value="' + index + '"]').prop('checked', true);
                            filterIndex = true;
                        } else {
                            filters['status'] = '3:4';
                        }
                    }else if( targetIdx == 'cvss' ){
                        if ( index != 'fixed' ) {
                            filters['status'] = '1:2';
                            filters['cvsses'] = [ index ];
                            filterBtnParent.find('[data-cvss][value="' + index + '"]').prop('checked', true);
                            filterIndex = true;
                        } else {
                            filters['status'] = '3:4';
                        }
                    }
                    const statusFilterBtn = Self.getfilterButton('status');
                    statusFilterBtn.parent().find('[data-name="status"]').removeClass(Self.selectedLinkClass);
                    statusFilterBtn.parent().find('[data-name="status"][data-value="' + filters['status'] + '"]')
                                            .addClass(Self.selectedLinkClass);
                    Self.resetPagesCheckedValues();
                    $('#pages_filters').val(JSON.stringify(filters));
                    Self.scrollToFilterButtonsArea = true;
                    $(`#${Self.dynamicSummaryTable.summaryTableId}`).bootstrapTable('refresh', { pageNumber: 1 });
                    const targetFilterButtons = [ statusFilterBtn ];
                    if( filterIndex ) targetFilterButtons.push(filterBtn);
                    targetFilterButtons.forEach( btn => {
                        btn.addClass(Self.filteringBtnClass).removeClass(Self.unFilteredBtnClass);
                    });
                });
            }
        });
    }

    //対応の状況分布のグラフ内クリックでのフィルタ
    enableGraphStatusFilter(selector){
        const Self = this;
        $(`${selector}`).each((idx, element) => {
            const elm = $(element);
            const href = elm.attr('xlink:href');
            if ( href && href.startsWith('#') ) {
                elm.on('click', function(e) {
                    e.preventDefault();
                    Self.resetAllFilters(false);
                    const index = $(this).attr('xlink:href').slice(1);
                    const filters = JSON.parse($('#pages_filters').val());
                    filters['status'] = index;
                    Self.scrollToFilterButtonsArea = true;
                    const statusFilterBtn = Self.getfilterButton('status');
                    statusFilterBtn.parent().find('[data-name="status"]').removeClass(Self.selectedLinkClass);
                    statusFilterBtn.parent().find('[data-name="status"][data-value="' + filters['status'] + '"]')
                                            .addClass(Self.selectedLinkClass);
                    Self.executeFilter(filters, 'status');
                });
            }
        });
    }

    //フィルタボタン以外での、要素クリックでのフィルタ。
    enableFilterClick(selector, itemName, func){
        const Self = this;
        const defaultFilters = Self.getDefaultFilters();

        $(`${selector}`).on('click', function(e) {
            e.preventDefault();
            const elm = $(this);
            const filters = JSON.parse($('#pages_filters').val());
            const filterBtn = Self.getfilterButton(itemName);
            func(elm, filters, filterBtn);
            Self.executeFilter(filters, itemName);
            return false;
        });
    }
};
