# -*- coding: utf-8 -*-

class CreateNewActionSyslog < ActiveRecord::Migration
  def change
    # syslogへの出力
    # VMのコンテナではsyslogサービスを起動しないため、リモートへの送信のみ
    create_table :sidfmd_rule_a_send_syslogs do |t|
      t.references :sidfmd_rule_action, foreign_key: true, index: true, null: false

      # 送信テスト用のコマンド
      # bundle exec ruby -e 'require "syslog_ruby" ; a = SyslogRuby::Logger.new("SIDfmVM", :LOCAL3, uri: "udp://172.17.0.1:514"); a.hostname = "sidfmvmsrv" ; a.info("test message")'

      # 出力されるメッセージの例
      # Aug 13 11:30:42 sidfmvmsrv SIDfmVM[10720]: test message in docker

      # プロトコル
      # 'udp' か 'tcp'
      # unix domain socketは利用不可（SIDfmVMのコンテナ内でsyslogを動作させないため)
      t.text :protocol, null: false, default: 'udp'

      # 送信先ホスト
      # ホスト名かIPアドレス
      # default値はdockerホストのIPアドレス
      t.text :target, null: false, default: '172.17.0.1'

      # 送信先ポート番号
      # default値は514(syslog)
      t.integer :port, null: false, default: 514

      # ホスト名
      # メッセージの"sidfmvmsrv"の部分に出力される内容を指定する。
      # ホスト名と同じチェックを行うこと。
      t.text :hostname, null: false, default: 'sidfmvmsrv'

      # 識別子
      # メッセージの"SIDfmVM"の部分に出力される内容を指定する。
      # 半角英数文字のみ許可。
      t.text :ident, null: false, default: 'SIDfmVM'

      # ログの種類を示す文字列(facility)
      # 受けたsyslogサービス側で、この情報を元にログファイルを分けたり、処理を
      # 行ったりする。
      # 以下の文字列のみ指定可能。
      # SyslogRuby::Facility.constants.find_all {|x| /^LOG/ !~ x.to_s }.map(&:to_s)
      t.text :facility, null: false, default: 'LOCAL3'

      # ログの深刻度を示す文字列(severity)
      # 受けたsyslogサービス側で、この情報を元にログファイルを分けたり、処理を
      # 行ったりする。
      # 以下の文字列のみ指定可能。
      # SyslogRuby::Severity.constants.find_all {|x| /^(LOG|WARNING)/ !~ x.to_s }.map(&:to_s)
      t.text :severity, null: false, default: 'INFO'

      # ログメッセージを示す文字列。
      # タグ文字利用可能。メール送信と同様に送信時にタグ文字を置換する。
      t.text :message, null: false, default: ''
    end

  end
end
